@@ -25,6 +25,7 @@
 #include <qdir.h>
 #include <qfile.h>
 #include <qlayout.h>
+#include <qptrlist.h>
 #include <qregexp.h>
 #include <qvbox.h>
 
@@ -52,7 +53,6 @@
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
 #include <kstdguiitem.h>
-#include <ktempfile.h>
 #include <kxmlguiclient.h>
 #include <ktoolbar.h>
 #include <libkdepim/addresseeview.h>
@@ -67,6 +67,7 @@
 #include "jumpbuttonbar.h"
 #include "kablock.h"
 #include "kabprefs.h"
+#include "kabtools.h"
 #include "kaddressbookservice.h"
 #include "kaddressbookiface.h"
 #include "ldapsearchdialog.h"
@@ -117,6 +118,8 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
                                 "X-IMAddress", "KADDRESSBOOK" );
   mAddressBook->addCustomField( i18n( "Anniversary" ), KABC::Field::Personal,
                                 "X-Anniversary", "KADDRESSBOOK" );
+  mAddressBook->addCustomField( i18n( "Blog" ), KABC::Field::Personal,
+                                "Blog", "KADDRESSBOOK" );
 
   mSearchManager = new KAB::SearchManager( mAddressBook, parent );
 
@@ -137,22 +140,27 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
            SLOT( setModified() ) );
   connect( mViewManager, SIGNAL( urlDropped( const KURL& ) ),
            mXXPortManager, SLOT( importVCard( const KURL& ) ) );
+  connect( mViewManager, SIGNAL( viewFieldsChanged() ),
+           SLOT( updateIncSearchWidget() ) );
   connect( mExtensionManager, SIGNAL( modified( const KABC::Addressee::List& ) ),
            this, SLOT( extensionModified( const KABC::Addressee::List& ) ) );
 
   connect( mXXPortManager, SIGNAL( modified() ),
            SLOT( setModified() ) );
 
-  connect( mJumpButtonBar, SIGNAL( jumpToLetter( const QStringList& ) ),
-           SLOT( incrementalJumpButtonSearch( const QStringList& ) ) );
+  connect( mJumpButtonBar, SIGNAL( jumpToLetter( const QString& ) ),
+           SLOT( incrementalJumpButtonSearch( const QString& ) ) );
   connect( mViewManager, SIGNAL( sortFieldChanged() ),
            mJumpButtonBar, SLOT( updateButtons() ) );
-  connect( mIncSearchWidget, SIGNAL( doReset() ),
-           mJumpButtonBar, SLOT( reset() ) );
 
   connect( mDetails, SIGNAL( highlightedMessage( const QString& ) ),
            SLOT( detailsHighlighted( const QString& ) ) );
 
+  connect( mIncSearchWidget, SIGNAL( scrollUp() ),
+           mViewManager, SLOT( scrollUp() ) );
+  connect( mIncSearchWidget, SIGNAL( scrollDown() ),
+           mViewManager, SLOT( scrollDown() ) );
+
   mAddressBookService = new KAddressBookService( this );
 
   mSearchManager->reload();
@@ -166,9 +174,6 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
 
 KABCore::~KABCore()
 {
-  saveSettings();
-  KABPrefs::instance()->writeConfig();
-
   mAddressBook->disconnect();
 
   mAddressBook = 0;
@@ -178,27 +183,28 @@ KABCore::~KABCore()
 
 void KABCore::restoreSettings()
 {
-  bool state = KABPrefs::instance()->mJumpButtonBarVisible;
+  bool state = KABPrefs::instance()->jumpButtonBarVisible();
   mActionJumpBar->setChecked( state );
   setJumpButtonBarVisible( state );
 
-  state = KABPrefs::instance()->mDetailsPageVisible;
+  state = KABPrefs::instance()->detailsPageVisible();
   mActionDetails->setChecked( state );
   setDetailsVisible( state );
 
   mViewManager->restoreSettings();
   mExtensionManager->restoreSettings();
 
-  mIncSearchWidget->setCurrentItem( KABPrefs::instance()->mCurrentIncSearchField );
+  updateIncSearchWidget();
+  mIncSearchWidget->setCurrentItem( KABPrefs::instance()->currentIncSearchField() );
 
-  QValueList<int> splitterSize = KABPrefs::instance()->mExtensionsSplitter;
+  QValueList<int> splitterSize = KABPrefs::instance()->extensionsSplitter();
   if ( splitterSize.count() == 0 ) {
     splitterSize.append( mDetailsSplitter->height() / 2 );
     splitterSize.append( mDetailsSplitter->height() / 2 );
   }
   mExtensionBarSplitter->setSizes( splitterSize );
 
-  splitterSize = KABPrefs::instance()->mDetailsSplitter;
+  splitterSize = KABPrefs::instance()->detailsSplitter();
   if ( splitterSize.count() == 0 ) {
     splitterSize.append( 360 );
     splitterSize.append( 260 );
@@ -209,16 +215,16 @@ void KABCore::restoreSettings()
 
 void KABCore::saveSettings()
 {
-  KABPrefs::instance()->mJumpButtonBarVisible = mActionJumpBar->isChecked();
-  KABPrefs::instance()->mDetailsPageVisible = mActionDetails->isChecked();
+  KABPrefs::instance()->setJumpButtonBarVisible( mActionJumpBar->isChecked() );
+  KABPrefs::instance()->setDetailsPageVisible( mActionDetails->isChecked() );
 
-  KABPrefs::instance()->mExtensionsSplitter = mExtensionBarSplitter->sizes();
-  KABPrefs::instance()->mDetailsSplitter = mDetailsSplitter->sizes();
+  KABPrefs::instance()->setExtensionsSplitter( mExtensionBarSplitter->sizes() );
+  KABPrefs::instance()->setDetailsSplitter( mDetailsSplitter->sizes() );
 
   mExtensionManager->saveSettings();
   mViewManager->saveSettings();
 
-  KABPrefs::instance()->mCurrentIncSearchField = mIncSearchWidget->currentItem();
+  KABPrefs::instance()->setCurrentIncSearchField( mIncSearchWidget->currentItem() );
 }
 
 KABC::AddressBook *KABCore::addressBook() const
@@ -274,9 +280,9 @@ QWidget *KABCore::widget() const
 KAboutData *KABCore::createAboutData()
 {
   KAboutData *about = new KAboutData( "kaddressbook", I18N_NOOP( "KAddressBook" ),
-                                      "3.3.2", I18N_NOOP( "The KDE Address Book" ),
+                                      "3.4", I18N_NOOP( "The KDE Address Book" ),
                                       KAboutData::License_GPL_V2,
-                                      I18N_NOOP( "(c) 1997-2004, The KDE PIM Team" ) );
+                                      I18N_NOOP( "(c) 1997-2005, The KDE PIM Team" ) );
   about->addAuthor( "Tobias Koenig", I18N_NOOP( "Current maintainer" ), "tokoe@kde.org" );
   about->addAuthor( "Don Sanders", I18N_NOOP( "Original author" ) );
   about->addAuthor( "Cornelius Schumacher",
@@ -323,6 +329,7 @@ void KABCore::setContactSelected( const QString &uid )
   mActionCopy->setEnabled( selected );
   mActionDelete->setEnabled( selected );
   mActionEditAddressee->setEnabled( selected );
+  mActionStoreAddresseeIn->setEnabled( selected );
   mActionMail->setEnabled( selected );
   mActionMailVCard->setEnabled( selected );
   mActionChat->setEnabled( selected && mKIMProxy && mKIMProxy->initialize() );
@@ -348,55 +355,9 @@ void KABCore::mailVCard()
     mailVCard( uids );
 }
 
-void KABCore::mailVCard( const QStringList& uids )
+void KABCore::mailVCard( const QStringList &uids )
 {
-  //QStringList urls;
-  KURL::List urls;
-
-  // Create a temp dir, so that we can put the files in it with proper names
-  KTempFile tempDir;
-  if ( tempDir.status() != 0 ) {
-    kdWarning() << strerror( tempDir.status() ) << endl;
-    return;
-  }
-
-  QString dirName = tempDir.name();
-  tempDir.unlink();
-  QDir().mkdir( dirName, true );
-
-  for( QStringList::ConstIterator it = uids.begin(); it != uids.end(); ++it ) {
-    KABC::Addressee a = mAddressBook->findByUid( *it );
-
-    if ( a.isEmpty() )
-      continue;
-
-    QString name = a.givenName().utf8() + "_" + a.familyName().utf8() + ".vcf";
-
-    QString fileName = dirName + "/" + name;
-
-    QFile outFile(fileName);
-    if ( outFile.open( IO_WriteOnly ) ) {  // file opened successfully
-      KABC::VCardConverter converter;
-      KABC::Addressee::List list;
-      list.append( a );
-      QString vcard = converter.createVCards( list, KABC::VCardConverter::v3_0 );
-
-      QTextStream t( &outFile );  // use a text stream
-      t.setEncoding( QTextStream::UnicodeUTF8 );
-      t << vcard;
-
-      outFile.close();
-
-      KURL url( fileName );
-      url.setFileEncoding( "UTF-8" );
-      urls.append( url );
-    }
-  }
-  kapp->invokeMailer( QString::null, QString::null, QString::null,
-                      QString::null,  // subject
-                      QString::null,  // body
-                      QString::null,
-                      urls.toStringList() );  // attachments
+  KABTools::mailVCards( uids, mAddressBook );
 }
 
 void KABCore::startChat()
@@ -434,8 +395,8 @@ void KABCore::deleteContacts( const QStringList &uids )
       ++it;
     }
 
-    if ( KMessageBox::warningContinueCancelList( mWidget, i18n( "Do you really want to delete these contacts?" ),
-                                         names, "", KGuiItem( i18n("&Delete"), "editdelete") ) == KMessageBox::Cancel )
+	    if ( KMessageBox::warningContinueCancelList( mWidget, i18n( "Do you really want to delete this contact?", "Do you really want to delete these %n contacts?", uids.count() ),
+		 names, "", KStdGuiItem::del() ) == KMessageBox::Cancel )
       return;
 
     PwDeleteCommand *command = new PwDeleteCommand( mAddressBook, uids );
@@ -502,7 +463,7 @@ void KABCore::mergeContacts()
   if ( list.count() < 2 )
     return;
 
-  KABC::Addressee addr = mergeContacts( list );
+  KABC::Addressee addr = KABTools::mergeContacts( list );
 
   KABC::Addressee::List::Iterator it = list.begin();
   ++it;
@@ -533,13 +494,25 @@ void KABCore::setWhoAmI()
 void KABCore::incrementalTextSearch( const QString& text )
 {
   setContactSelected( QString::null );
-  mSearchManager->search( text, mIncSearchWidget->currentField() );
+  mSearchManager->search( text, mIncSearchWidget->currentFields() );
 }
 
-void KABCore::incrementalJumpButtonSearch( const QStringList& characters )
+void KABCore::incrementalJumpButtonSearch( const QString& character )
 {
-  setContactSelected( QString::null );
-  mSearchManager->setJumpButtonFilter( characters, mViewManager->currentSortField() );
+  mViewManager->setSelected( QString::null, false );
+
+  KABC::AddresseeList list = mSearchManager->contacts();
+  KABC::Field *field = mViewManager->currentSortField();
+  if ( field ) {
+    list.sortByField( field );
+    KABC::AddresseeList::ConstIterator it;
+    for ( it = list.begin(); it != list.end(); ++it ) {
+      if ( field->value( *it ).startsWith( character, false ) ) {
+        mViewManager->setSelected( (*it).uid(), true );
+        return;
+      }
+    }
+  }
 }
 
 void KABCore::setModified()
@@ -563,15 +536,13 @@ bool KABCore::modified() const
 void KABCore::contactModified( const KABC::Addressee &addr )
 {
   Command *command = 0;
-  QString uid;
 
   // check if it exists already
   KABC::Addressee origAddr = mAddressBook->findByUid( addr.uid() );
-  if ( origAddr.isEmpty() )
+  if ( origAddr.isEmpty() ) {
     command = new PwNewCommand( mAddressBook, addr );
-  else {
+  } else {
     command = new PwEditCommand( mAddressBook, origAddr, addr );
-    uid = addr.uid();
   }
 
   UndoStack::instance()->push( command );
@@ -585,22 +556,7 @@ void KABCore::newContact()
 {
   AddresseeEditorDialog *dialog = 0;
 
-  QPtrList<KABC::Resource> kabcResources = mAddressBook->resources();
-
-  QPtrList<KRES::Resource> kresResources;
-  QPtrListIterator<KABC::Resource> it( kabcResources );
-  KABC::Resource *resource;
-  while ( ( resource = it.current() ) != 0 ) {
-    ++it;
-    if ( !resource->readOnly() ) {
-      KRES::Resource *res = static_cast<KRES::Resource*>( resource );
-      if ( res )
-        kresResources.append( res );
-    }
-  }
-
-  KRES::Resource *res = KRES::SelectDialog::getResource( kresResources, mWidget );
-  resource = static_cast<KABC::Resource*>( res );
+  KABC::Resource* resource = requestResource( mWidget );
 
   if ( resource ) {
     KABC::Addressee addr;
@@ -678,8 +634,9 @@ void KABCore::editContact( const QString &uid )
     if ( !dialog ) {
 
       if ( !addr.resource()->readOnly() )
-        if ( !KABLock::self( mAddressBook )->lock( addr.resource() ) )
+        if ( !KABLock::self( mAddressBook )->lock( addr.resource() ) ) {
           return;
+        }
 
       dialog = createAddresseeEditorDialog( mWidget );
 
@@ -693,36 +650,57 @@ void KABCore::editContact( const QString &uid )
   }
 }
 
+void KABCore::storeContactIn( const QString &uid )
+{
+  // First, locate the contact entry
+  QStringList uidList;
+  if ( uid.isNull() ) {
+    uidList = mViewManager->selectedUids();
+  } else {
+    uidList << uid;
+  }
+  KABC::Resource *resource = requestResource( mWidget );
+  if ( !resource ) return;
+  KABLock::self( mAddressBook )->lock( resource );
+  QStringList::Iterator it( uidList.begin() );
+  while ( it != uidList.end() ) {
+    KABC::Addressee addr = mAddressBook->findByUid( *it++ );
+    if ( !addr.isEmpty() ) {
+      KABC::Addressee newAddr( addr );
+      // We need to set a new uid, otherwise the insert below is
+      // ignored. This is bad for syncing, but unavoidable, afaiks
+      newAddr.setUid( KApplication::randomString( 10 ) );
+      newAddr.setResource( resource );
+      addressBook()->insertAddressee( newAddr );
+      KABLock::self( mAddressBook )->lock( addr.resource() );
+      addressBook()->removeAddressee( addr );
+      KABLock::self( mAddressBook )->unlock( addr.resource() );
+    }
+  }
+  KABLock::self( mAddressBook )->unlock( resource );
+}
+
 void KABCore::save()
 {
-  KABC::StdAddressBook *b = dynamic_cast<KABC::StdAddressBook*>( mAddressBook );
-  if ( b ) {
-    if ( !b->save() ) {
-      QString text = i18n( "There was an error while attempting to save the "
-    	                   "address book. Please check that no other application "
-                           "is using it." );
-
-      KMessageBox::error( mWidget, text, i18n( "Unable to Save" ) );
-    } else {
-      setModified( false );
-    }
-  } else {
-    // FIXME: Handle locking properly, i.e. get the ticket before doing the
-    // first change to the addressbook and don't give up the ticket in case of a
-    // save error without asking the user.
-    KABC::Ticket *ticket = mAddressBook->requestSaveTicket();
+  QPtrList<KABC::Resource> resources = mAddressBook->resources();
+  QPtrListIterator<KABC::Resource> it( resources );
+  while ( it.current() && !it.current()->readOnly() ) {
+    KABC::Ticket *ticket = mAddressBook->requestSaveTicket( it.current() );
     if ( ticket ) {
       if ( !mAddressBook->save( ticket ) ) {
-        KMessageBox::error( mWidget, i18n("Error saving address book.") );
+        KMessageBox::error( mWidget,
+                            i18n( "<qt>Unable to save address book <b>%1</b>.</qt>" ).arg( it.current()->resourceName() ) );
         mAddressBook->releaseSaveTicket( ticket );
       } else {
         setModified( false );
       }
     } else {
       KMessageBox::error( mWidget,
-                          i18n("Unable to get access for saving the address "
-                               "book.") );
+                          i18n( "<qt>Unable to get access for saving the address book <b>%1</b>.</qt>" )
+                          .arg( it.current()->resourceName() ) );
     }
+
+    ++it;
   }
 }
 
@@ -792,7 +770,7 @@ QString KABCore::getNameByPhone( const QString &phone )
 
   bool found = false;
   QString ownerName = "";
-  KABC::AddressBook::Iterator iter;
+  KABC::AddressBook::ConstIterator iter;
   KABC::PhoneNumber::List::Iterator phoneIter;
   KABC::PhoneNumber::List phoneList;
   for ( iter = mAddressBook->begin(); !found && ( iter != mAddressBook->end() ); ++iter ) {
@@ -815,7 +793,7 @@ void KABCore::openLDAPDialog()
   if ( !KProtocolInfo::isKnownProtocol( KURL( "ldap://localhost" ) ) ) {
     KMessageBox::error( mWidget, i18n( "Your KDE installation is missing LDAP "
                                        "support, please ask your administrator or distributor for more information." ),
-                        i18n( "No LDAP IO slave available" ) );
+                        i18n( "No LDAP IO Slave Available" ) );
     return;
   }
 
@@ -851,6 +829,9 @@ void KABCore::configure()
 void KABCore::print()
 {
   KPrinter printer;
+  printer.setDocName( i18n( "Address Book" ) );
+  printer.setDocFileName( "addressbook" );
+
   if ( !printer.setup( mWidget, i18n("Print Addresses") ) )
     return;
 
@@ -876,8 +857,8 @@ void KABCore::showContactsAddress( const QString &addrUid )
   if ( addr.isEmpty() )
     return;
 
-  KABC::Address::List list = addr.addresses();
-  KABC::Address::List::Iterator it;
+  const KABC::Address::List list = addr.addresses();
+  KABC::Address::List::ConstIterator it;
   for ( it = list.begin(); it != list.end(); ++it )
     if ( (*it).id() == addrUid ) {
       LocationMap::instance()->showAddress( *it );
@@ -888,14 +869,32 @@ void KABCore::showContactsAddress( const QString &addrUid )
 void KABCore::configurationChanged()
 {
   mExtensionManager->reconfigure();
-  mSearchManager->reconfigure();
   mViewManager->refreshView();
 }
 
+bool KABCore::queryClose()
+{
+  saveSettings();
+  KABPrefs::instance()->writeConfig();
+
+  QPtrList<KABC::Resource> resources = mAddressBook->resources();
+  QPtrListIterator<KABC::Resource> it( resources );
+  while ( it.current() ) {
+    it.current()->close();
+    ++it;
+  }
+
+  return true;
+}
+
 void KABCore::addressBookChanged()
 {
   mJumpButtonBar->updateButtons();
   mSearchManager->reload();
+  mViewManager->setSelected( QString::null, false );
+  setContactSelected( QString::null );
+
+  updateCategories();
 }
 
 AddresseeEditorDialog *KABCore::createAddresseeEditorDialog( QWidget *parent,
@@ -913,15 +912,15 @@ AddresseeEditorDialog *KABCore::createAddresseeEditorDialog( QWidget *parent,
 
 void KABCore::slotEditorDestroyed( const QString &uid )
 {
-  mEditorDict.remove( uid );
+  AddresseeEditorDialog *dialog = mEditorDict.take( uid );
 
-  KABC::Addressee addr = mAddressBook->findByUid( uid );
+  KABC::Addressee addr = dialog->addressee();
 
-  QApplication::setOverrideCursor( Qt::waitCursor );
-
-  KABLock::self( mAddressBook )->unlock( addr.resource() );
-
-  QApplication::restoreOverrideCursor();
+  if ( !addr.resource()->readOnly() ) {
+    QApplication::setOverrideCursor( Qt::waitCursor );
+    KABLock::self( mAddressBook )->unlock( addr.resource() );
+    QApplication::restoreOverrideCursor();
+  }
 }
 
 void KABCore::initGUI()
@@ -936,11 +935,7 @@ void KABCore::initGUI()
 
   mFilterSelectionWidget = new FilterSelectionWidget( searchTB , "kde toolbar widget" );
 
-  QHBoxLayout *hbox = new QHBoxLayout( mWidget, 0, 0 );
-
   mDetailsSplitter = new QSplitter( mWidget );
-  hbox->addWidget( mDetailsSplitter );
-
   topLayout->addWidget( searchTB );
   topLayout->addWidget( mDetailsSplitter );
 
@@ -972,7 +967,7 @@ void KABCore::initGUI()
   detailsLayout->addWidget( mJumpButtonBar );
   detailsLayout->setStretchFactor( mJumpButtonBar, 1 );
 
-  topLayout->setStretchFactor( hbox, 1 );
+  topLayout->setStretchFactor( mDetailsSplitter, 1 );
 
   mXXPortManager = new XXPortManager( this, mWidget );
 
@@ -1039,6 +1034,13 @@ void KABCore::initActions()
                                actionCollection(), "edit_delete" );
   mActionDelete->setWhatsThis( i18n( "Delete all selected contacts." ) );
 
+
+  mActionStoreAddresseeIn = new KAction( i18n( "St&ore Contact In..." ), "kaddressbook", 0,
+                                      this, SLOT( storeContactIn() ),
+                                      actionCollection(), "edit_store_in" );
+  mActionStoreAddresseeIn->setWhatsThis( i18n( "Store a contact in a different Addressbook<p>You will be presented with a dialog where you can select a new storage place for this contact." ) );
+
+
   mActionUndo->setEnabled( false );
   mActionRedo->setEnabled( false );
 
@@ -1046,17 +1048,14 @@ void KABCore::initActions()
   mActionJumpBar = new KToggleAction( i18n( "Show Jump Bar" ), "next", 0,
                                       actionCollection(), "options_show_jump_bar" );
   mActionJumpBar->setWhatsThis( i18n( "Toggle whether the jump button bar shall be visible." ) );
+  mActionJumpBar->setCheckedState( i18n( "Hide Jump Bar") );
   connect( mActionJumpBar, SIGNAL( toggled( bool ) ), SLOT( setJumpButtonBarVisible( bool ) ) );
 
   mActionDetails = new KToggleAction( i18n( "Show Details" ), 0, 0,
                                       actionCollection(), "options_show_details" );
   mActionDetails->setWhatsThis( i18n( "Toggle whether the details page shall be visible." ) );
-  connect( mActionDetails, SIGNAL( toggled( bool ) ), SLOT( setDetailsVisible( bool ) ) );
-
-#if KDE_IS_VERSION(3,2,90)
-  mActionJumpBar->setCheckedState( i18n( "Hide Jump Bar") );
   mActionDetails->setCheckedState( i18n( "Hide Details") );
-#endif
+  connect( mActionDetails, SIGNAL( toggled( bool ) ), SLOT( setDetailsVisible( bool ) ) );
 
   if ( mIsPart )
     action = new KAction( i18n( "&Configure KAddressBook..." ), "configure", 0,
@@ -1082,6 +1081,12 @@ void KABCore::initActions()
                                    "edit_set_categories" );
   mActionCategories->setWhatsThis( i18n( "Set the categories for all selected contacts." ) );
 
+  KAction *clearLocation = new KAction( i18n( "Clear Search Bar" ),
+					QApplication::reverseLayout() ? "clear_left" : "locationbar_erase",
+					CTRL+Key_L, this, SLOT( slotClearSearchBar() ), actionCollection(), "clear_search" );
+  clearLocation->setWhatsThis( i18n( "Clear Search Bar<p>"
+				     "Clears the content of the quick search bar." ) );
+
   clipboardDataChanged();
 
   connect( UndoStack::instance(), SIGNAL( changed() ), SLOT( updateActionMenu() ) );
@@ -1114,141 +1119,45 @@ void KABCore::updateActionMenu()
   mActionRedo->setEnabled( !redo->isEmpty() );
 }
 
-KABC::Addressee KABCore::mergeContacts( const KABC::Addressee::List &list )
+void KABCore::updateIncSearchWidget()
 {
-  if ( list.count() == 0 ) //emtpy
-    return KABC::Addressee();
-  else if ( list.count() == 1 ) // nothing to merge
-    return list.first();
+  mIncSearchWidget->setViewFields( mViewManager->viewFields() );
+}
 
-  KABC::Addressee masterAddressee = list.first();
+void KABCore::updateCategories()
+{
+  QStringList categories( allCategories() );
+  categories.sort();
 
-  KABC::Addressee::List::ConstIterator contactIt = list.begin();
-  for ( ++contactIt; contactIt != list.end(); ++contactIt ) {
-    // ADR + LABEL
-    KABC::Address::List addresses = (*contactIt).addresses();
-    KABC::Address::List masterAddresses = masterAddressee.addresses();
-    KABC::Address::List::Iterator addrIt ;
-    for ( addrIt = addresses.begin(); addrIt != addresses.end(); ++addrIt ) {
-      if ( !masterAddresses.contains( *addrIt ) )
-        masterAddressee.insertAddress( *addrIt );
+  const QStringList customCategories( KABPrefs::instance()->customCategories() );
+  QStringList::ConstIterator it;
+  for ( it = customCategories.begin(); it != customCategories.end(); ++it ) {
+    if ( categories.find( *it ) == categories.end() ) {
+      categories.append( *it );
     }
-
-    if ( masterAddressee.birthday().isNull() && !(*contactIt).birthday().isNull() )
-      masterAddressee.setBirthday( (*contactIt).birthday() );
-
-
-    // CATEGORIES
-    QStringList::Iterator it;
-    QStringList categories = (*contactIt).categories();
-    QStringList masterCategories = masterAddressee.categories();
-    QStringList newCategories( masterCategories );
-    for ( it = categories.begin(); it != categories.end(); ++it )
-      if ( !masterCategories.contains( *it ) )
-        newCategories.append( *it );
-    masterAddressee.setCategories( newCategories );
-
-    // CLASS
-    if ( !masterAddressee.secrecy().isValid() && (*contactIt).secrecy().isValid() )
-      masterAddressee.setSecrecy( (*contactIt).secrecy() );
-
-    // EMAIL
-    QStringList emails = (*contactIt).emails();
-    QStringList masterEmails = masterAddressee.emails();
-    for ( it = emails.begin(); it != emails.end(); ++it )
-      if ( !masterEmails.contains( *it ) )
-        masterAddressee.insertEmail( *it, false );
-
-    // FN
-    if ( masterAddressee.formattedName().isEmpty() && !(*contactIt).formattedName().isEmpty() )
-      masterAddressee.setFormattedName( (*contactIt).formattedName() );
-
-    // GEO
-    if ( !masterAddressee.geo().isValid() && (*contactIt).geo().isValid() )
-      masterAddressee.setGeo( (*contactIt).geo() );
-
-/*
-  // KEY
-  // LOGO
-*/
-
-    // MAILER
-    if ( masterAddressee.mailer().isEmpty() && !(*contactIt).mailer().isEmpty() )
-      masterAddressee.setMailer( (*contactIt).mailer() );
-
-    // N
-    if ( masterAddressee.assembledName().isEmpty() && !(*contactIt).assembledName().isEmpty() )
-      masterAddressee.setNameFromString( (*contactIt).assembledName() );
-
-    // NICKNAME
-    if ( masterAddressee.nickName().isEmpty() && !(*contactIt).nickName().isEmpty() )
-      masterAddressee.setNickName( (*contactIt).nickName() );
-
-    // NOTE
-    if ( masterAddressee.note().isEmpty() && !(*contactIt).note().isEmpty() )
-      masterAddressee.setNote( (*contactIt).note() );
-
-    // ORG
-    if ( masterAddressee.organization().isEmpty() && !(*contactIt).organization().isEmpty() )
-      masterAddressee.setOrganization( (*contactIt).organization() );
-
-/*
-  // PHOTO
-*/
-
-    // PROID
-    if ( masterAddressee.productId().isEmpty() && !(*contactIt).productId().isEmpty() )
-      masterAddressee.setProductId( (*contactIt).productId() );
-
-    // REV
-    if ( masterAddressee.revision().isNull() && !(*contactIt).revision().isNull() )
-      masterAddressee.setRevision( (*contactIt).revision() );
-
-    // ROLE
-    if ( masterAddressee.role().isEmpty() && !(*contactIt).role().isEmpty() )
-      masterAddressee.setRole( (*contactIt).role() );
-
-    // SORT-STRING
-    if ( masterAddressee.sortString().isEmpty() && !(*contactIt).sortString().isEmpty() )
-      masterAddressee.setSortString( (*contactIt).sortString() );
-
-/*
-  // SOUND
-*/
-
-    // TEL
-    KABC::PhoneNumber::List phones = (*contactIt).phoneNumbers();
-    KABC::PhoneNumber::List masterPhones = masterAddressee.phoneNumbers();
-    KABC::PhoneNumber::List::ConstIterator phoneIt;
-    for ( phoneIt = phones.begin(); phoneIt != phones.end(); ++phoneIt )
-      if ( !masterPhones.contains( *it ) )
-        masterAddressee.insertPhoneNumber( *it );
-
-    // TITLE
-    if ( masterAddressee.title().isEmpty() && !(*contactIt).title().isEmpty() )
-      masterAddressee.setTitle( (*contactIt).title() );
-
-    // TZ
-    if ( !masterAddressee.timeZone().isValid() && (*contactIt).timeZone().isValid() )
-      masterAddressee.setTimeZone( (*contactIt).timeZone() );
-
-    // UID // ignore UID
-
-    // URL
-    if ( masterAddressee.url().isEmpty() && !(*contactIt).url().isEmpty() )
-      masterAddressee.setUrl( (*contactIt).url() );
-
-    // X-
-    QStringList customs = (*contactIt).customs();
-    QStringList masterCustoms = masterAddressee.customs();
-    QStringList newCustoms( masterCustoms );
-    for ( it = customs.begin(); it != customs.end(); ++it )
-      if ( !masterCustoms.contains( *it ) )
-        newCustoms.append( *it );
-    masterAddressee.setCustoms( newCustoms );
   }
 
-  return masterAddressee;
+  KABPrefs::instance()->mCustomCategories = categories;
+  KABPrefs::instance()->writeConfig();
+
+  if ( mCategoryEditDialog )
+    mCategoryEditDialog->reload();
+}
+
+QStringList KABCore::allCategories() const
+{
+  QStringList categories, allCategories;
+  QStringList::ConstIterator catIt;
+
+  KABC::AddressBook::ConstIterator it;
+  for ( it = mAddressBook->begin(); it != mAddressBook->end(); ++it ) {
+    categories = (*it).categories();
+    for ( catIt = categories.begin(); catIt != categories.end(); ++catIt )
+      if ( !allCategories.contains( *catIt ) )
+        allCategories.append( *catIt );
+  }
+
+  return allCategories;
 }
 
 void KABCore::setCategories()
@@ -1261,9 +1170,6 @@ void KABCore::setCategories()
     connect( mCategorySelectDialog, SIGNAL( editCategories() ), SLOT( editCategories() ) );
   }
 
-  QStringList selected = mCategorySelectDialog->selectedCategories();
-  mCategorySelectDialog->setCategories();
-  mCategorySelectDialog->setSelected( selected );
   mCategorySelectDialog->show();
   mCategorySelectDialog->raise();
 }
@@ -1305,13 +1211,19 @@ void KABCore::editCategories()
   if ( mCategoryEditDialog == 0 ) {
     mCategoryEditDialog = new KPIM::CategoryEditDialog( KABPrefs::instance(), mWidget );
     connect( mCategoryEditDialog, SIGNAL( categoryConfigChanged() ),
-             SLOT( setCategories() ) );
+             mCategorySelectDialog, SLOT( updateCategoryConfig() ) );
   }
 
   mCategoryEditDialog->show();
   mCategoryEditDialog->raise();
 }
 
+void KABCore::slotClearSearchBar()
+{
+  mIncSearchWidget->clear();
+  mIncSearchWidget->setFocus();
+}
+
 bool KABCore::handleCommandLine( KAddressBookIface* iface )
 {
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
