@@ -37,6 +37,7 @@ static KCmdLineOptions options[] =
 	{ "ack-confirm", I18N_NOOP("Prompt for confirmation when alarm is acknowledged"), 0 },
 	{ "A", 0, 0 },
 	{ "attach <url>", I18N_NOOP("Attach file to email (repeat as needed)"), 0 },
+	{ "auto-close", I18N_NOOP("Auto-close alarm window after --late-cancel period"), 0 },
 	{ "bcc", I18N_NOOP("Blind copy email to self"), 0 },
 	{ "b", 0, 0 },
 	{ "beep", I18N_NOOP("Beep when message is displayed"), 0 },
@@ -48,20 +49,19 @@ static KCmdLineOptions options[] =
 	{ "colorfg <color>", I18N_NOOP("Message foreground color (name or hex 0xRRGGBB)"), 0 },
 	{ "calendarURL <url>", I18N_NOOP("URL of calendar file"), 0 },
 	{ "cancelEvent <eventID>", I18N_NOOP("Cancel alarm with the specified event ID"), 0 },
+	{ "d", 0, 0 },
+	{ "disable", I18N_NOOP("Disable the alarm"), 0 },
 	{ "e", 0, 0 },
 	{ "exec <commandline>", I18N_NOOP("Execute a shell command line"), 0 },
 	{ "f", 0, 0 },
 	{ "file <url>", I18N_NOOP("File to display"), 0 },
+	{ "F", 0, 0 },
+	{ "from-id <ID>", I18N_NOOP("KMail identity to use as sender of email"), 0 },
 	{ "handleEvent <eventID>", I18N_NOOP("Trigger or cancel alarm with the specified event ID"), 0 },
 	{ "i", 0, 0 },
 	{ "interval <period>", I18N_NOOP("Interval between alarm repetitions"), 0 },
 	{ "l", 0, 0 },
-	{ "late-cancel", I18N_NOOP("Cancel alarm if it cannot be triggered on time"), 0 },
-#if 0
-	{ "auto-close", I18N_NOOP("Auto-close alarm window after --late-cancel period"), 0 },
-	{ "disable", I18N_NOOP("Disable the alarm"), 0 },
 	{ "late-cancel <period>", I18N_NOOP("Cancel alarm if more than 'period' late when triggered"), "1" },
-#endif
 	{ "L", 0, 0 },
 	{ "login", I18N_NOOP("Repeat alarm at every login"), 0 },
 	{ "m", 0, 0 },
