@@ -1,7 +1,7 @@
 /*
  *  messagewin.cpp  -  displays an alarm message
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 #include "kalarm.h"
 
+#include <stdlib.h>
 #include <string.h>
 
 #include <qfile.h>
@@ -43,6 +44,7 @@
 #include <kiconloader.h>
 #include <kdialog.h>
 #include <ktextbrowser.h>
+#include <kglobalsettings.h>
 #include <kmimetype.h>
 #include <kmessagebox.h>
 #include <kwin.h>
@@ -58,6 +60,7 @@
 #include <arts/kartsserver.h>
 #include <arts/kplayobjectfactory.h>
 #include <arts/kplayobject.h>
+#include <dcopclient.h>
 #endif
 #include <kdebug.h>
 
@@ -65,12 +68,22 @@
 #include "deferdlg.h"
 #include "functions.h"
 #include "kalarmapp.h"
+#include "mainwindow.h"
 #include "preferences.h"
 #include "synchtimer.h"
 #include "messagewin.moc"
 
 using namespace KCal;
 
+#ifndef WITHOUT_ARTS
+static const char* KMIX_APP_NAME    = "kmix";
+static const char* KMIX_DCOP_OBJECT = "Mixer0";
+#endif
+
+// The delay for enabling message window buttons if a zero delay is
+// configured, i.e. the windows are placed far from the cursor.
+static const int proximityButtonDelay = 1000;    // (milliseconds)
+static const int proximityMultiple = 10;         // multiple of button height distance from cursor for proximity
 
 // A text label widget which can be scrolled and copied with the mouse
 class MessageText : public QTextEdit
@@ -141,15 +154,17 @@ MessageWin::MessageWin(const KAEvent& event, const KAAlarm& alarm, bool reschedu
 	  mEvent(event),
 	  mDeferButton(0),
 	  mSilenceButton(0),
+	  mDeferDlg(0),
 	  mFlags(event.flags()),
+	  mLateCancel(event.lateCancel()),
 	  mErrorWindow(false),
 	  mNoPostAction(false),
 	  mRecreating(false),
 	  mBeep(event.beep()),
 	  mRescheduleEvent(reschedule_event),
 	  mShown(false),
-	  mDeferClosing(false),
-	  mDeferDlgShowing(false)
+	  mPositioning(false),
+	  mDeferClosing(false)
 {
 	kdDebug(5950) << "MessageWin::MessageWin(event)" << endl;
 	setAutoSaveSettings(QString::fromLatin1("MessageWin"));     // save window sizes etc.
@@ -177,13 +192,14 @@ MessageWin::MessageWin(const KAEvent& event, const DateTime& alarmDateTime, cons
 	  mEvent(event),
 	  mDeferButton(0),
 	  mSilenceButton(0),
+	  mDeferDlg(0),
 	  mErrorWindow(true),
 	  mNoPostAction(true),
 	  mRecreating(false),
 	  mRescheduleEvent(false),
 	  mShown(false),
-	  mDeferClosing(false),
-	  mDeferDlgShowing(false)
+	  mPositioning(false),
+	  mDeferClosing(false)
 {
 	kdDebug(5950) << "MessageWin::MessageWin(errmsg)" << endl;
 	initView();
@@ -199,13 +215,14 @@ MessageWin::MessageWin()
 	  mArtsDispatcher(0),
 	  mPlayObject(0),
 	  mSilenceButton(0),
+	  mDeferDlg(0),
 	  mErrorWindow(false),
 	  mNoPostAction(true),
 	  mRecreating(false),
 	  mRescheduleEvent(false),
 	  mShown(false),
-	  mDeferClosing(false),
-	  mDeferDlgShowing(false)
+	  mPositioning(false),
+	  mDeferClosing(false)
 {
 	kdDebug(5950) << "MessageWin::MessageWin()\n";
 	mWindowList.append(this);
@@ -299,7 +316,7 @@ void MessageWin::initView()
 				bool dir = false;
 				QString tmpFile;
 				KURL url(mMessage);
-				if (KIO::NetAccess::download(url, tmpFile))
+				if (KIO::NetAccess::download(url, tmpFile, MainWindow::mainMainWindow()))
 				{
 					QFile qfile(tmpFile);
 					QFileInfo info(qfile);
@@ -324,7 +341,7 @@ void MessageWin::initView()
 				if (!opened)
 				{
 					// File couldn't be opened
-					bool exists = KIO::NetAccess::exists(url);
+					bool exists = KIO::NetAccess::exists(url, true, MainWindow::mainMainWindow());
 					mErrorMsgs += dir ? i18n("File is a folder") : exists ? i18n("Failed to open file") : i18n("File not found");
 				}
 				break;
@@ -454,14 +471,14 @@ void MessageWin::initView()
 	int gridIndex = 1;
 
 	// Close button
-	KPushButton* okButton = new KPushButton(KStdGuiItem::close(), topWidget);
+	mOkButton = new KPushButton(KStdGuiItem::close(), topWidget);
 	// Prevent accidental acknowledgement of the message if the user is typing when the window appears
-	okButton->clearFocus();
-	okButton->setFocusPolicy(QWidget::ClickFocus);    // don't allow keyboard selection
-	okButton->setFixedSize(okButton->sizeHint());
-	connect(okButton, SIGNAL(clicked()), SLOT(close()));
-	grid->addWidget(okButton, 0, gridIndex++, AlignHCenter);
-	QWhatsThis::add(okButton, i18n("Acknowledge the alarm"));
+	mOkButton->clearFocus();
+	mOkButton->setFocusPolicy(QWidget::ClickFocus);    // don't allow keyboard selection
+	mOkButton->setFixedSize(mOkButton->sizeHint());
+	connect(mOkButton, SIGNAL(clicked()), SLOT(close()));
+	grid->addWidget(mOkButton, 0, gridIndex++, AlignHCenter);
+	QWhatsThis::add(mOkButton, i18n("Acknowledge the alarm"));
 
 	if (!mNoDefer)
 	{
@@ -474,6 +491,8 @@ void MessageWin::initView()
 		QWhatsThis::add(mDeferButton,
 		      i18n("Defer the alarm until later.\n"
 		           "You will be prompted to specify when the alarm should be redisplayed."));
+
+		setDeferralLimit(mEvent);    // ensure that button is disabled when alarm can't be deferred any more
 	}
 
 #ifndef WITHOUT_ARTS
@@ -496,14 +515,21 @@ void MessageWin::initView()
 	// KAlarm button
 	KIconLoader iconLoader;
 	QPixmap pixmap = iconLoader.loadIcon(QString::fromLatin1(kapp->aboutData()->appName()), KIcon::MainToolbar);
-	QPushButton* button = new QPushButton(topWidget);
-	button->setPixmap(pixmap);
-	button->setFixedSize(button->sizeHint());
-	connect(button, SIGNAL(clicked()), SLOT(displayMainWindow()));
-	grid->addWidget(button, 0, gridIndex++, AlignHCenter);
+	mKAlarmButton = new QPushButton(topWidget);
+	mKAlarmButton->setPixmap(pixmap);
+	mKAlarmButton->setFixedSize(mKAlarmButton->sizeHint());
+	connect(mKAlarmButton, SIGNAL(clicked()), SLOT(displayMainWindow()));
+	grid->addWidget(mKAlarmButton, 0, gridIndex++, AlignHCenter);
 	QString actKAlarm = i18n("Activate %1").arg(kapp->aboutData()->programName());
-	QToolTip::add(button, actKAlarm);
-	QWhatsThis::add(button, actKAlarm);
+	QToolTip::add(mKAlarmButton, actKAlarm);
+	QWhatsThis::add(mKAlarmButton, actKAlarm);
+
+	// Disable all buttons initially, to prevent accidental clicking on if they happen to be
+	// under the mouse just as the window appears.
+	mOkButton->setEnabled(false);
+	if (mDeferButton)
+		mDeferButton->setEnabled(false);
+	mKAlarmButton->setEnabled(false);
 
 	topLayout->activate();
 	setMinimumSize(QSize(grid->sizeHint().width() + 2*KDialog::marginHint(), sizeHint().height()));
@@ -525,7 +551,7 @@ void MessageWin::setRemainingTextDay()
 	if (days == 0  &&  !mDateTime.isDateOnly())
 	{
 		// The alarm is due today, so start refreshing every minute
-		DailyTimer::disconnect(this);
+		DailyTimer::disconnect(this, SLOT(setRemainingTextDay()));
 		setRemainingTextMinute();
 		MinuteTimer::connect(this, SLOT(setRemainingTextMinute()));   // update every minute
 	}
@@ -585,7 +611,7 @@ void MessageWin::saveProperties(KConfig* config)
 		if (mAudioRepeat  &&  mSilenceButton  &&  mSilenceButton->isEnabled())
 		{
 			// Only need to restart sound file playing if it's being repeated
-			config->writeEntry(QString::fromLatin1("AudioFile"), mAudioFile);
+			config->writePathEntry(QString::fromLatin1("AudioFile"), mAudioFile);
 			config->writeEntry(QString::fromLatin1("Volume"), static_cast<int>(mVolume * 100));
 		}
 #endif
@@ -617,7 +643,7 @@ void MessageWin::readProperties(KConfig* config)
 	bool dateOnly  = config->readBoolEntry(QString::fromLatin1("DateOnly"));
 	mDateTime.set(dt, dateOnly);
 #ifndef WITHOUT_ARTS
-	mAudioFile     = config->readEntry(QString::fromLatin1("AudioFile"));
+	mAudioFile     = config->readPathEntry(QString::fromLatin1("AudioFile"));
 	mVolume        = static_cast<float>(config->readNumEntry(QString::fromLatin1("Volume"))) / 100;
 	if (!mAudioFile.isEmpty())
 		mAudioRepeat = true;
@@ -675,14 +701,16 @@ void MessageWin::playAudio()
 
 /******************************************************************************
 *  Play the audio file.
+*  Called asynchronously to avoid delaying the display of the message.
 */
 void MessageWin::slotPlayAudio()
 {
 #ifndef WITHOUT_ARTS
 	// First check that it exists, to avoid possible crashes if the filename is badly specified
+	MainWindow* mmw = MainWindow::mainMainWindow();
 	KURL url(mAudioFile);
-	if (!url.isValid()  ||  !KIO::NetAccess::exists(url)
-	||  !KIO::NetAccess::download(url, mLocalAudioFile))
+	if (!url.isValid()  ||  !KIO::NetAccess::exists(url, true, mmw)
+	||  !KIO::NetAccess::download(url, mLocalAudioFile, mmw))
 	{
 		kdError(5950) << "MessageWin::playAudio(): Open failure: " << mAudioFile << endl;
 		KMessageBox::error(this, i18n("Cannot open audio file:\n%1").arg(mAudioFile), kapp->aboutData()->programName());
@@ -698,6 +726,19 @@ void MessageWin::slotPlayAudio()
 		initAudio(true);
 		if (!mPlayObject->object().isNull())
 			checkAudioPlay();
+#if KDE_VERSION >= 308
+		if (!mUsingKMix)
+		{
+			// Output error message now that everything else has been done.
+			// (Outputting it earlier would delay things until it is acknowledged.)
+			bool kmixRunning = kapp->dcopClient()->isApplicationRegistered(KMIX_APP_NAME);
+			QString reason = kmixRunning ? i18n("Error accessing KMix") : i18n("KMix not running");
+			const char* reasonName = kmixRunning ? "KMixError" : "KMixNotRun";
+			KMessageBox::information(this, i18n("Unable to set master volume\n(%1)").arg(reason),
+			                         QString::null, QString::fromLatin1(reasonName));
+			kdWarning(5950) << "Unable to set master volume (KMix " << (kmixRunning ? "DCOP error" : "not running") << ")\n";
+		}
+#endif
 	}
 #endif
 }
@@ -713,8 +754,26 @@ void MessageWin::initAudio(bool firstTime)
 	KDE::PlayObjectFactory factory(sserver);
 	mPlayObject = factory.createPlayObject(mLocalAudioFile, true);
 	if (firstTime)
-		mOldVolume = sserver.outVolume().scaleFactor();    // save volume for restoration afterwards
-	sserver.outVolume().scaleFactor(mVolume >= 0 ? mVolume : 1);
+	{
+		// Get the current master volume from KMix
+		int vol = getKMixVolume();
+		if (vol >= 0)
+		{
+			mOldVolume = vol;    // success
+			mUsingKMix = true;
+		}
+		else
+		{
+			// Can't use KMix to set the master volume, so just adjust
+			// within the current master volume.
+			mOldVolume = sserver.outVolume().scaleFactor();    // save volume for restoration afterwards
+			mUsingKMix = false;
+		}
+	}
+	if (!mUsingKMix)
+		sserver.outVolume().scaleFactor(mVolume >= 0 ? mVolume : 1);
+	else if (mVolume >= 0)
+		setKMixVolume(static_cast<int>(mVolume * 100));
 	mSilenceButton->setEnabled(true);
 	mPlayed = false;
 	connect(mPlayObject, SIGNAL(playObjectCreated()), SLOT(checkAudioPlay()));
@@ -724,8 +783,10 @@ void MessageWin::initAudio(bool firstTime)
 #endif
 
 /******************************************************************************
-*  Called to check whether the audio file playing has completed, and if not to
-*  wait a bit longer.
+*  Called when the audio file has loaded and is ready to play, or on a timer
+*  when play is expected to have completed.
+*  If it is ready to play, start playing it (for the first time or repeated).
+*  If play has not yet completed, wait a bit longer.
 */
 void MessageWin::checkAudioPlay()
 {
@@ -734,8 +795,11 @@ void MessageWin::checkAudioPlay()
 		return;
 	if (mPlayObject->state() == Arts::posIdle)
 	{
+		// The file has loaded and is ready to play, or play has completed
 		if (mPlayedOnce  &&  !mAudioRepeat)
 			return;
+
+		// Start playing the file, either for the first time or again
 		kdDebug(5950) << "MessageWin::checkAudioPlay(): start\n";
 		if (!mPlayedOnce)
 		{
@@ -796,14 +860,25 @@ void MessageWin::stopPlay()
 	{
 		// Restore the sound volume to what it was before the sound file
 		// was played, provided that nothing else has modified it since.
-		KArtsServer aserver;
-		Arts::StereoVolumeControl svc = aserver.server().outVolume();
-		float currentVolume = svc.scaleFactor();
-		float eventVolume = mVolume;
-		if (eventVolume < 0)
-			eventVolume = 1;
-		if (currentVolume == eventVolume)
-			svc.scaleFactor(mOldVolume);
+		if (!mUsingKMix)
+		{
+			KArtsServer aserver;
+			Arts::StereoVolumeControl svc = aserver.server().outVolume();
+			float currentVolume = svc.scaleFactor();
+			float eventVolume = mVolume;
+			if (eventVolume < 0)
+				eventVolume = 1;
+			if (currentVolume == eventVolume)
+				svc.scaleFactor(mOldVolume);
+		}
+		else if (mVolume >= 0)
+		{
+			int eventVolume = static_cast<int>(mVolume * 100);
+			int currentVolume = getKMixVolume();
+			// Volume returned isn't always exactly equal to volume set
+			if (currentVolume < 0  ||  abs(currentVolume - eventVolume) < 5)
+				setKMixVolume(static_cast<int>(mOldVolume));
+		}
 	}
 	delete mPlayObject;      mPlayObject = 0;
 	delete mArtsDispatcher;  mArtsDispatcher = 0;
@@ -814,26 +889,103 @@ void MessageWin::stopPlay()
 #endif
 }
 
+#ifndef WITHOUT_ARTS
 /******************************************************************************
-*  Re-output any required audio notification, and reschedule the alarm in the
-*  calendar file.
+*  Get the master volume from KMix.
+*  Reply < 0 if failure.
+*/
+int MessageWin::getKMixVolume()
+{
+	if (!runKMix())     // start KMix if it isn't already running
+		return -1;
+	QByteArray  data, replyData;
+	QCString    replyType;
+	QDataStream arg(data, IO_WriteOnly);
+	if (!kapp->dcopClient()->call(mKMixName, KMIX_DCOP_OBJECT, "masterVolume()", data, replyType, replyData)
+	||  replyType != "int")
+		return -1;
+	int result;
+	QDataStream reply(replyData, IO_ReadOnly);
+	reply >> result;
+	return (result >= 0) ? result : 0;
+}
+
+/******************************************************************************
+*  Set the master volume using KMix.
+*/
+void MessageWin::setKMixVolume(int percent)
+{
+	if (!mUsingKMix)
+		return;
+	if (!runKMix())     // start KMix if it isn't already running
+		return;
+	QByteArray  data;
+	QDataStream arg(data, IO_WriteOnly);
+	arg << percent;
+	if (!kapp->dcopClient()->send(mKMixName, KMIX_DCOP_OBJECT, "setMasterVolume(int)", data))
+		kdError(5950) << "MessageWin::setKMixVolume(): kmix dcop call failed\n";
+}
+
+/******************************************************************************
+*  Start KMix if it isn't already running.
+*  Reply = true if KMix is now running.
+*/
+bool MessageWin::runKMix()
+{
+	if (mKMixName.isEmpty())
+		mKMixName = KMIX_APP_NAME;
+	if (!kapp->dcopClient()->isApplicationRegistered(mKMixName))
+	{
+		// KMix is not already running, so start it
+		if (KApplication::startServiceByDesktopName(QString::fromLatin1(KMIX_APP_NAME), QString::null, &mKMixError, &mKMixName))
+			return false;
+	}
+	return true;
+}
+#endif
+
+/******************************************************************************
+*  Raise the alarm window, re-output any required audio notification, and
+*  reschedule the alarm in the calendar file.
 */
 void MessageWin::repeat(const KAAlarm& alarm)
 {
+	if (mDeferDlg)
+	{
+		// Cancel any deferral dialogue so that the user notices something's going on,
+		// and also because the deferral time limit will have changed.
+		delete mDeferDlg;
+		mDeferDlg = 0;
+	}
 	const Event* kcalEvent = mEventID.isNull() ? 0 : AlarmCalendar::activeCalendar()->event(mEventID);
 	if (kcalEvent)
 	{
 		mAlarmType = alarm.type();    // store new alarm type for use if it is later deferred
-		if (!mDeferDlgShowing  ||  Preferences::instance()->modalMessages())
+		if (!mDeferDlg  ||  Preferences::instance()->modalMessages())
 		{
 			raise();
 			playAudio();
 		}
 		KAEvent event(*kcalEvent);
+		mDeferButton->setEnabled(true);
+		setDeferralLimit(event);    // ensure that button is disabled when alarm can't be deferred any more
 		theApp()->alarmShowing(event, mAlarmType, mDateTime);
 	}
 }
 
+/******************************************************************************
+*  Display the window.
+*  If windows are being positioned away from the mouse cursor, it is initially
+*  positioned at the top left to slightly reduce the number of times the
+*  windows need to be moved in showEvent().
+*/
+void MessageWin::show()
+{
+	if (Preferences::instance()->messageButtonDelay() == 0)
+		move(0, 0);
+	MainWindowBase::show();
+}
+
 /******************************************************************************
 *  Called when the window is shown.
 *  The first time, output any required audio notification, and reschedule or
@@ -842,19 +994,112 @@ void MessageWin::repeat(const KAAlarm& alarm)
 void MessageWin::showEvent(QShowEvent* se)
 {
 	MainWindowBase::showEvent(se);
-	if (!mShown  &&  !mErrorWindow)
+	if (!mShown)
 	{
-		QSize s = sizeHint();     // fit the window round the message
-		if (mAction == KAEvent::FILE  &&  !mErrorMsgs.count())
-			KAlarm::readConfigWindowSize("FileMessage", s);
-		resize(s);
-		playAudio();
-		if (mRescheduleEvent)
-			theApp()->alarmShowing(mEvent, mAlarmType, mDateTime);
+		if (mErrorWindow)
+			enableButtons();    // don't bother repositioning error messages
+		else
+		{
+			QSize s = sizeHint();     // fit the window round the message
+			if (mAction == KAEvent::FILE  &&  !mErrorMsgs.count())
+				KAlarm::readConfigWindowSize("FileMessage", s);
+			resize(s);
+
+			mButtonDelay = Preferences::instance()->messageButtonDelay() * 1000;
+			if (!mButtonDelay)
+			{
+				/* Try to ensure that the window can't accidentally be acknowledged
+				 * by the user clicking the mouse just as it appears.
+				 * To achieve this, move the window so that the OK button is as far away
+				 * from the cursor as possible. If the buttons are still too close to the
+				 * cursor, disable the buttons for a short time.
+				 * N.B. This can't be done in show(), since the geometry of the window
+				 *      is not known until it is displayed. Unfortunately by moving the
+				 *      window in showEvent(), a flicker is unavoidable.
+				 *      See the Qt documentation on window geometry for more details.
+				 */
+				QRect desk = KGlobalSettings::desktopGeometry(this);
+				QPoint cursor = QCursor::pos();
+				QRect frame = frameGeometry();
+				QRect rect  = geometry();
+				// Find the offsets from the outside of the frame to the edges of the OK button
+				QRect button(mOkButton->mapToParent(QPoint(0, 0)), mOkButton->mapToParent(mOkButton->rect().bottomRight()));
+				int buttonLeft   = button.left() + rect.left() - frame.left();
+				int buttonRight  = width() - button.right() + frame.right() - rect.right();
+				int buttonTop    = button.top() + rect.top() - frame.top();
+				int buttonBottom = height() - button.bottom() + frame.bottom() - rect.bottom();
+
+				int centrex = (desk.width() + buttonLeft - buttonRight) / 2;
+				int centrey = (desk.height() + buttonTop - buttonBottom) / 2;
+				int x = (cursor.x() < centrex) ? desk.right() - frame.width() : desk.left();
+				int y = (cursor.y() < centrey) ? desk.bottom() - frame.height() : desk.top();
+
+				// Find the enclosing rectangle for the new button positions
+				// and check if the cursor is too near
+				QRect buttons = mOkButton->geometry().unite(mKAlarmButton->geometry());
+				buttons.moveBy(rect.left() + x - frame.left(), rect.top() + y - frame.top());
+				int minDistance = proximityMultiple * mOkButton->height();
+				if ((abs(cursor.x() - buttons.left()) < minDistance
+				  || abs(cursor.x() - buttons.right()) < minDistance)
+				&&  (abs(cursor.y() - buttons.top()) < minDistance
+				  || abs(cursor.y() - buttons.bottom()) < minDistance))
+					mButtonDelay = proximityButtonDelay;    // too near - disable buttons initially
+
+				if (x != frame.left()  ||  y != frame.top())
+				{
+					mPositioning = true;
+					move(x, y);
+				}
+			}
+			if (!mPositioning)
+				displayComplete();    // play audio, etc.
+		}
 		mShown = true;
 	}
 }
 
+/******************************************************************************
+*  Called when the window has been moved.
+*/
+void MessageWin::moveEvent(QMoveEvent* e)
+{
+	MainWindowBase::moveEvent(e);
+	if (mPositioning)
+	{
+		// The window has just been initially positioned
+		mPositioning = false;
+		displayComplete();    // play audio, etc.
+	}
+}
+
+/******************************************************************************
+*  Called when the window has been displayed properly (in its correct position),
+*  to play sounds and reschedule the event.
+*/
+void MessageWin::displayComplete()
+{
+	playAudio();
+	if (mRescheduleEvent)
+		theApp()->alarmShowing(mEvent, mAlarmType, mDateTime);
+
+	// Enable the window's buttons either now or after the configured delay
+	if (mButtonDelay > 0)
+		QTimer::singleShot(mButtonDelay, this, SLOT(enableButtons()));
+	else
+		enableButtons();
+}
+
+/******************************************************************************
+*  Enable the window's buttons.
+*/
+void MessageWin::enableButtons()
+{
+	mOkButton->setEnabled(true);
+	mKAlarmButton->setEnabled(true);
+	if (mDeferButton)
+		mDeferButton->setEnabled(true);
+}
+
 /******************************************************************************
 *  Called when the window's size has changed (before it is painted).
 */
@@ -904,28 +1149,71 @@ void MessageWin::closeEvent(QCloseEvent* ce)
 	MainWindowBase::closeEvent(ce);
 }
 
+/******************************************************************************
+* Set up to disable the defer button when the deferral limit is reached.
+*/
+void MessageWin::setDeferralLimit(const KAEvent& event)
+{
+	if (mDeferButton)
+	{
+		// Ensure that defer button is disabled when the deferral limit is reached
+		mDeferLimit = event.deferralLimit().dateTime();
+		DailyTimer::connect(this, SLOT(checkDeferralLimit()));   // check every day
+		checkDeferralLimit();
+	}
+}
+
+/******************************************************************************
+* Check whether the deferral limit has been reached.
+* If so, disable the Defer button.
+* N.B. Ideally, just a single QTimer::singleShot() call would be made to disable
+*      the defer button at the corret time. But for a 32-bit integer, the
+*      milliseconds parameter overflows in about 25 days, so instead a daily
+*      check is done until the day when the deferral limit is reached, followed
+*      by a non-overflowing QTimer::singleShot() call.
+*/
+void MessageWin::checkDeferralLimit()
+{
+	if (!mDeferButton  ||  !mDeferLimit.isValid())
+		return;
+	int n = QDate::currentDate().daysTo(mDeferLimit.date());
+	if (n > 0)
+		return;
+	DailyTimer::disconnect(this, SLOT(checkDeferralLimit()));
+	if (n == 0)
+	{
+		// The deferral limit will be reached today
+		n = QTime::currentTime().secsTo(mDeferLimit.time());
+		if (n > 0)
+		{
+			QTimer::singleShot(n * 1000, this, SLOT(checkDeferralLimit()));
+			return;
+		}
+	}
+	mDeferButton->setEnabled(false);
+}
+
 /******************************************************************************
 *  Called when the Defer... button is clicked.
 *  Displays the defer message dialog.
 */
 void MessageWin::slotDefer()
 {
-	DeferAlarmDlg deferDlg(i18n("Defer Alarm"), QDateTime::currentDateTime().addSecs(60),
-	                       false, this, "deferDlg");
-	deferDlg.setLimit(mEventID);
-	mDeferDlgShowing = true;
+	mDeferDlg = new DeferAlarmDlg(i18n("Defer Alarm"), QDateTime::currentDateTime().addSecs(60),
+	                              false, this, "deferDlg");
+	mDeferDlg->setLimit(mEventID);
 	if (!Preferences::instance()->modalMessages())
 		lower();
-	if (deferDlg.exec() == QDialog::Accepted)
+	if (mDeferDlg->exec() == QDialog::Accepted)
 	{
-		DateTime dateTime = deferDlg.getDateTime();
+		DateTime dateTime = mDeferDlg->getDateTime();
 		const Event* kcalEvent = mEventID.isNull() ? 0 : AlarmCalendar::activeCalendar()->event(mEventID);
 		if (kcalEvent)
 		{
 			// The event still exists in the calendar file.
 			KAEvent event(*kcalEvent);
-			event.defer(dateTime, (mAlarmType & KAAlarm::REMINDER_ALARM), true);
-			KAlarm::updateEvent(event, 0);
+			bool repeat = event.defer(dateTime, (mAlarmType & KAAlarm::REMINDER_ALARM), true);
+			KAlarm::updateEvent(event, 0, true, !repeat);
 		}
 		else
 		{
@@ -939,7 +1227,7 @@ void MessageWin::slotDefer()
 			else
 			{
 				// The event doesn't exist any more !?!, so create a new one
-				event.set(dateTime.dateTime(), mMessage, mBgColour, mFgColour, mFont, mAction, mFlags);
+				event.set(dateTime.dateTime(), mMessage, mBgColour, mFgColour, mFont, mAction, mLateCancel, mFlags);
 				event.setAudioFile(mAudioFile, mVolume);
 				event.setArchive();
 				event.setEventID(mEventID);
@@ -963,7 +1251,8 @@ void MessageWin::slotDefer()
 	}
 	else
 		raise();
-	mDeferDlgShowing = false;
+	delete mDeferDlg;
+	mDeferDlg = 0;
 }
 
 /******************************************************************************
