@@ -46,6 +46,7 @@
 #include "listjob.h"
 using KMail::ListJob;
 #include "kmailicalifaceimpl.h"
+#include "kmsearchpattern.h"
 
 #include <klocale.h>
 #include <kconfig.h>
@@ -207,8 +208,6 @@ void FolderStorage::quiet(bool beQuiet)
 
 //-----------------------------------------------------------------------------
 
-// Needed to use QSortedList in reduceSize()
-
 /** Compare message's date. This is useful for message sorting */
 int operator<( KMMsgBase & m1, KMMsgBase & m2 )
 {
@@ -273,7 +272,7 @@ bool FolderStorage::canAddMsgNow(KMMessage* aMsg, int* aIndex_ret)
     connect(job, SIGNAL(messageRetrieved(KMMessage*)),
             SLOT(reallyAddMsg(KMMessage*)));
     job->start();
-    aMsg->setTransferInProgress(TRUE);
+    aMsg->setTransferInProgress( true );
     return FALSE;
   }
   return TRUE;
@@ -285,7 +284,8 @@ void FolderStorage::reallyAddMsg(KMMessage* aMsg)
 {
   if (!aMsg) // the signal that is connected can call with aMsg=0
     return;
-  aMsg->setTransferInProgress(FALSE);
+  aMsg->setTransferInProgress( false );
+  aMsg->setComplete( true );
   KMFolder *aFolder = aMsg->parent();
   int index;
   ulong serNum = aMsg->getMsgSerNum();
@@ -303,6 +303,7 @@ void FolderStorage::reallyAddMsg(KMMessage* aMsg)
 //-----------------------------------------------------------------------------
 void FolderStorage::reallyAddCopyOfMsg(KMMessage* aMsg)
 {
+  if ( !aMsg ) return; // messageRetrieved(0) is always possible
   aMsg->setParent( 0 );
   aMsg->setTransferInProgress( false );
   addMsg( aMsg );
@@ -363,12 +364,7 @@ void FolderStorage::removeMsg(int idx, bool)
   --mTotalMsgs;
 
   QString msgIdMD5 = mb->msgIdMD5();
-  QString strippedSubjMD5 = mb->strippedSubjectMD5();
-  if (strippedSubjMD5.isEmpty()) {
-     mb->initStrippedSubjectMD5();
-     strippedSubjMD5 = mb->strippedSubjectMD5();
-  }
-  emit msgRemoved(idx, msgIdMD5, strippedSubjMD5);
+  emit msgRemoved( idx, msgIdMD5 );
   emit msgRemoved( folder() );
 }
 
@@ -399,12 +395,7 @@ KMMessage* FolderStorage::take(int idx)
   setDirty( true );
   needsCompact=true; // message is taken from here - needs to be compacted
   QString msgIdMD5 = msg->msgIdMD5();
-  QString strippedSubjMD5 = msg->strippedSubjectMD5();
-  if (strippedSubjMD5.isEmpty()) {
-     msg->initStrippedSubjectMD5();
-     strippedSubjMD5 = msg->strippedSubjectMD5();
-  }
-  emit msgRemoved(idx, msgIdMD5, strippedSubjMD5);
+  emit msgRemoved( idx, msgIdMD5 );
   emit msgRemoved( folder() );
 
   return msg;
@@ -480,7 +471,7 @@ KMMessage* FolderStorage::readTemporaryMsg(int idx)
     msg = new KMMessage(*(KMMessage*)mb);
     msg->setMsgSerNum(sernum);
   } else {
-    // ## Those three lines need to be moved to a virtual method for KMFolderSearch, like readMsg
+    // ## Those two lines need to be moved to a virtual method for KMFolderSearch, like readMsg
     msg = new KMMessage(*(KMMsgInfo*)mb);
     msg->setMsgSerNum(sernum); // before fromDwString so that readyToShow uses the right sernum
     msg->fromDwString(getDwString(idx));
@@ -575,10 +566,13 @@ int FolderStorage::moveMsg(QPtrList<KMMessage> msglist, int* aIndex_ret)
   if (msgParent)
     msgParent->open();
 
+  QValueList<int> index;
   open();
-  //FIXME : is it always imap ?
-  int rc = static_cast<KMFolderImap*>(this)->addMsg(msglist, aIndex_ret); //yuck: Don
+  int rc = addMsg(msglist, index);
   close();
+  // FIXME: we want to have a QValueList to pass it back, so change this method
+  if ( !index.isEmpty() )
+    aIndex_ret = &index.first();
 
   if (msgParent)
     msgParent->close();
@@ -675,7 +669,9 @@ int FolderStorage::rename(const QString& newName, KMFolderDir *newParent)
   // delete the old entry as we get two entries with the same ID otherwise
   KMKernel::config()->deleteGroup( oldConfigString );
 
+  emit locationChanged( oldLoc, newLoc );
   emit nameChanged();
+  kmkernel->folderMgr()->contentsChanged();
   return rc;
 }
 
@@ -838,6 +834,7 @@ void FolderStorage::readConfig()
   int type = config->readNumEntry( "ContentsType", 0 );
   if ( type < 0 || type > KMail::ContentsTypeLast ) type = 0;
   setContentsType( static_cast<KMail::FolderContentsType>( type ) );
+  mContentsTypeChanged = false;
 
   if( folder() ) folder()->readConfig( config );
 }
@@ -847,7 +844,8 @@ void FolderStorage::writeConfig()
 {
   KConfig* config = KMKernel::config();
   KConfigGroupSaver saver(config, "Folder-" + folder()->idString());
-  config->writeEntry("UnreadMsgs", mGuessedUnreadMsgs == -1 ? mUnreadMsgs : -1);
+  config->writeEntry("UnreadMsgs", 
+      mGuessedUnreadMsgs == -1 ? mUnreadMsgs : mGuessedUnreadMsgs);
   config->writeEntry("TotalMsgs", mTotalMsgs);
   config->writeEntry("Compactable", mCompactable);
   config->writeEntry("ContentsType", mContentsType);
@@ -1022,4 +1020,60 @@ void FolderStorage::setContentsType( KMail::FolderContentsType type )
   }
 }
 
+//-----------------------------------------------------------------------------
+void FolderStorage::search( KMSearchPattern* pattern )
+{
+  mSearchPattern = pattern;
+  mCurrentSearchedMsg = 0;
+  slotProcessNextSearchBatch();
+}
+
+void FolderStorage::slotProcessNextSearchBatch()
+{
+  QValueList<Q_UINT32> matchingSerNums;
+  int end = ( count() - mCurrentSearchedMsg > 100 ) ? 100+mCurrentSearchedMsg : count();
+  for ( int i = mCurrentSearchedMsg; i < end; ++i )
+  {
+    Q_UINT32 serNum = kmkernel->msgDict()->getMsgSerNum( folder(), i );
+    if ( mSearchPattern->matches( serNum ) )
+      matchingSerNums.append( serNum );
+  }
+  mCurrentSearchedMsg = end;
+  bool complete = ( end == count() ) ? true : false;
+  emit searchResult( folder(), matchingSerNums, mSearchPattern, complete );
+  if ( !complete )
+    QTimer::singleShot( 0, this, SLOT(slotProcessNextSearchBatch()) );
+}
+
+//-----------------------------------------------------------------------------
+void FolderStorage::search( KMSearchPattern* pattern, Q_UINT32 serNum )
+{
+  Q_UINT32 mySerNum = 0;
+  if ( pattern->matches( serNum ) )
+    mySerNum = serNum;
+
+  emit searchDone( folder(), mySerNum, pattern );
+}
+
+//-----------------------------------------------------------------------------
+int FolderStorage::addMsg( QPtrList<KMMessage>& msgList, QValueList<int>& index_ret )
+{
+  int ret = 0;
+  int index;
+  for ( QPtrListIterator<KMMessage> it( msgList ); *it; ++it )
+  {
+    int aret = addMsg( *it, &index );
+    index_ret << index;
+    if ( aret != 0 ) // error condition
+      ret = aret;
+  }
+  return ret;
+}
+
+//-----------------------------------------------------------------------------
+bool FolderStorage::isMoveable() const
+{
+  return ( folder()->isSystemFolder() ) ? false : true;
+}
+
 #include "folderstorage.moc"
