@@ -98,9 +98,6 @@ public:
   virtual void setImapFolder(KMFolderCachedImap *);
   KMFolderCachedImap* imapFolder() const { return mFolder; }
 
-  bool isProgressDialogEnabled() const { return mProgressDialogEnabled; }
-  void setProgressDialogEnabled( bool enable ) { mProgressDialogEnabled = enable; }
-
   virtual void readConfig( /*const*/ KConfig/*Base*/ & config );
   virtual void writeConfig( KConfig/*Base*/ & config ) /*const*/;
 
@@ -174,6 +171,10 @@ public:
    * Returns the root folder of this account
    */
   virtual FolderStorage* const rootFolder() const;
+  
+  /** return if the account passed the annotation test  */
+  bool annotationCheckPassed(){ return mAnnotationCheckPassed;};
+  void setAnnotationCheckPassed( bool a ){ mAnnotationCheckPassed = a; };
 
 protected:
   friend class KMAcctMgr;
@@ -183,7 +184,7 @@ protected slots:
   /** new-mail-notification for the current folder (is called via folderComplete) */
   void postProcessNewMail(KMFolderCachedImap*, bool);
 
-  void slotProgressItemCanceled( ProgressItem* );
+  void slotProgressItemCanceled( KPIM::ProgressItem* );
 
   virtual void slotCheckQueuedFolders();
 
@@ -194,10 +195,10 @@ private:
 private:
   QPtrList<CachedImapJob> mJobList;
   KMFolderCachedImap *mFolder;
-  bool mProgressDialogEnabled;
   QStringList mDeletedFolders; // folders deleted in this session
   QStringList mPreviouslyDeletedFolders; // folders deleted in a previous session
   QMap<QString, RenamedFolder> mRenamedFolders;
+  bool mAnnotationCheckPassed;
 };
 
 #endif /*KMAcctCachedImap_h*/
