@@ -42,7 +42,7 @@
 
 #include <qtextcodec.h>
 
-#include <libkdepim/email.h>
+#include <libemailfunctions/email.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
 #include <kio/netaccess.h>
@@ -56,6 +56,8 @@
 #include <kbookmarkmanager.h>
 #include <kstandarddirs.h>
 #include <ktempfile.h>
+#include <kimproxy.h>
+#include <kuserprofile.h>
 #include "actionscheduler.h"
 using KMail::ActionScheduler;
 #include "mailinglist-magic.h"
@@ -67,9 +69,11 @@ using KMail::ActionScheduler;
 #include "kmfolderimap.h"
 #include "kmfoldermgr.h"
 #include "kmheaders.h"
+#include "headeritem.h"
 #include "kmmainwidget.h"
 #include "kmmsgdict.h"
 #include "kmsender.h"
+#include "kmmsgpartdlg.h"
 #include "undostack.h"
 #include "kcursorsaver.h"
 #include "partNode.h"
@@ -81,11 +85,15 @@ using KMail::MailSourceViewer;
 #include "kmreadermainwin.h"
 #include "secondarywindow.h"
 using KMail::SecondaryWindow;
-#include "kimproxy.h"
+#include <redirectdialog.h>
+using KMail::RedirectDialog;
+#include <libkdepim/kfileio.h>
 
 #include "progressmanager.h"
 using KPIM::ProgressManager;
 using KPIM::ProgressItem;
+#include <kmime_mdn.h>
+using namespace KMime;
 
 #include "broadcaststatus.h"
 
@@ -500,7 +508,7 @@ KMMailtoOpenAddrBookCommand::KMMailtoOpenAddrBookCommand( const KURL &url,
 KMCommand::Result KMMailtoOpenAddrBookCommand::execute()
 {
   QString addr = KMMessage::decodeMailtoUrl( mUrl.path() );
-  KAddrBookExternal::openEmail( KPIM::getEmailAddr(addr), addr ,
+  KAddrBookExternal::openEmail( KPIM::getEmailAddress(addr), addr ,
                                 parentWidget() );
 
   return OK;
@@ -621,14 +629,20 @@ KMCommand::Result KMEditMsgCommand::execute()
 }
 
 
-KMShowMsgSrcCommand::KMShowMsgSrcCommand( KMMessage *msg, bool fixedFont )
-  : mFixedFont( fixedFont ), mMsg ( msg )
+KMShowMsgSrcCommand::KMShowMsgSrcCommand( QWidget *parent,
+  KMMessage *msg, bool fixedFont )
+  :KMCommand( parent, msg ), mFixedFont( fixedFont )
 {
+  // remember complete state
+  mMsgWasComplete = msg->isComplete();
 }
 
-void KMShowMsgSrcCommand::start()
+KMCommand::Result KMShowMsgSrcCommand::execute()
 {
-  QString str = mMsg->codec()->toUnicode( mMsg->asString() );
+  KMMessage *msg = retrievedMessage();
+  if ( msg->isComplete() && !mMsgWasComplete )
+    msg->notify(); // notify observers as msg was transfered
+  QString str = msg->codec()->toUnicode( msg->asString() );
 
   MailSourceViewer *viewer = new MailSourceViewer(); // deletes itself upon close
   viewer->setCaption( i18n("Message as Plain Text") );
@@ -648,6 +662,8 @@ void KMShowMsgSrcCommand::start()
                   2*QApplication::desktop()->geometry().height()/3);
   }
   viewer->show();
+
+  return OK;
 }
 
 static KURL subjectToUrl( const QString & subject ) {
@@ -659,12 +675,21 @@ static KURL subjectToUrl( const QString & subject ) {
 KMSaveMsgCommand::KMSaveMsgCommand( QWidget *parent, KMMessage * msg )
   : KMCommand( parent ),
     mMsgListIndex( 0 ),
+    mStandAloneMessage( 0 ),
     mOffset( 0 ),
     mTotalSize( msg ? msg->msgSize() : 0 )
 {
   if ( !msg ) return;
   setDeletesItself( true );
-  mMsgList.append( msg->getMsgSerNum() );
+  // If the mail has a serial number, operate on sernums, if it does not
+  // we need to work with the pointer, but can be reasonably sure it won't
+  // go away, since it'll be an encapsulated message or one that was opened
+  // from an .eml file.
+  if ( msg->getMsgSerNum() != 0 ) {
+    mMsgList.append( msg->getMsgSerNum() );
+  } else {
+    mStandAloneMessage = msg;
+  }
   mUrl = subjectToUrl( msg->cleanSubject() );
 }
 
@@ -672,6 +697,7 @@ KMSaveMsgCommand::KMSaveMsgCommand( QWidget *parent,
                                     const QPtrList<KMMsgBase> &msgList )
   : KMCommand( parent ),
     mMsgListIndex( 0 ),
+    mStandAloneMessage( 0 ),
     mOffset( 0 ),
     mTotalSize( 0 )
 {
@@ -756,36 +782,44 @@ void KMSaveMsgCommand::slotSaveDataReq()
       }
     }
   } else {
-    // No more messages. Tell the putjob we are done.
-    QByteArray data = QByteArray();
-    mJob->sendAsyncData( data );
+    if ( mStandAloneMessage ) {
+      // do the special case of a standalone message
+      slotMessageRetrievedForSaving( mStandAloneMessage );
+      mStandAloneMessage = 0;
+    } else {
+      // No more messages. Tell the putjob we are done.
+      QByteArray data = QByteArray();
+      mJob->sendAsyncData( data );
+    }
   }
 }
 
 void KMSaveMsgCommand::slotMessageRetrievedForSaving(KMMessage *msg)
 {
-  QCString str( msg->mboxMessageSeparator() );
-  str += KMFolderMbox::escapeFrom( msg->asString() );
-  str += "\n";
-  msg->setTransferInProgress(false);
+  if ( msg ) {
+    QCString str( msg->mboxMessageSeparator() );
+    str += KMFolderMbox::escapeFrom( msg->asString() );
+    str += "\n";
+    msg->setTransferInProgress(false);
 
-  mData = str;
-  mData.resize(mData.size() - 1);
-  mOffset = 0;
-  QByteArray data;
-  int size;
-  // Unless it is great than 64 k send the whole message. kio buffers for us.
-  if( mData.size() > (unsigned int) MAX_CHUNK_SIZE )
-    size = MAX_CHUNK_SIZE;
-  else
-    size = mData.size();
+    mData = str;
+    mData.resize(mData.size() - 1);
+    mOffset = 0;
+    QByteArray data;
+    int size;
+    // Unless it is great than 64 k send the whole message. kio buffers for us.
+    if( mData.size() > (unsigned int) MAX_CHUNK_SIZE )
+      size = MAX_CHUNK_SIZE;
+    else
+      size = mData.size();
 
-  data.duplicate( mData, size );
-  mJob->sendAsyncData( data );
-  mOffset += size;
+    data.duplicate( mData, size );
+    mJob->sendAsyncData( data );
+    mOffset += size;
+  }
   ++mMsgListIndex;
   // Get rid of the message.
-  if (msg->parent()) {
+  if ( msg && msg->parent() && msg->getMsgSerNum() ) {
     int idx = -1;
     KMFolder * p = 0;
     kmkernel->msgDict()->getLocation( msg, &p, &idx );
@@ -882,7 +916,7 @@ void KMOpenMsgCommand::slotResult( KIO::Job *job )
       startOfMessage = mMsgString.find( '\n' );
       if ( startOfMessage == -1 ) {
         KMessageBox::sorry( parentWidget(),
-                            i18n( "The file doesn't contain a message." ) );
+                            i18n( "The file does not contain a message." ) );
         setResult( Failed );
         emit completed( this );
         // Emulate closing of a secondary window so that KMail exits in case it
@@ -910,7 +944,7 @@ void KMOpenMsgCommand::slotResult( KIO::Job *job )
     // check whether we have a message ( no headers => this isn't a message )
     if ( dwMsg->Headers().NumFields() == 0 ) {
       KMessageBox::sorry( parentWidget(),
-                          i18n( "The file doesn't contain a message." ) );
+                          i18n( "The file does not contain a message." ) );
       delete dwMsg; dwMsg = 0;
       setResult( Failed );
       emit completed( this );
@@ -1161,14 +1195,16 @@ KMCommand::Result KMForwardCommand::execute()
   }
 
   // forward a single message at most.
-
   KMMessage *msg = msgList.getFirst();
   if ( !msg || !msg->codec() )
     return Failed;
 
   KCursorSaver busy(KBusyPtr::busy());
-  win = new KMComposeWin(msg->createForward());
-  win->setCharset(msg->codec()->mimeName(), TRUE);
+  KMMessage *fwdMsg = msg->createForward();
+
+  win = new KMComposeWin( fwdMsg );
+  win->setCharset( fwdMsg->codec()->mimeName(), true );
+  win->setBody( QString::fromUtf8( msg->createForwardBody() ) );
   win->show();
 
   return OK;
@@ -1247,53 +1283,44 @@ KMRedirectCommand::KMRedirectCommand( QWidget *parent,
 
 KMCommand::Result KMRedirectCommand::execute()
 {
-  //TODO: move KMMessage::createRedirect to here
-  KMComposeWin *win;
   KMMessage *msg = retrievedMessage();
   if ( !msg || !msg->codec() )
     return Failed;
 
-  KCursorSaver busy(KBusyPtr::busy());
-  win = new KMComposeWin();
-  win->setMsg(msg->createRedirect(), FALSE);
-  win->setCharset(msg->codec()->mimeName());
-  win->show();
+  RedirectDialog dlg( parentWidget(), "redirect", true,
+                      kmkernel->msgSender()->sendImmediate() );
+  if (dlg.exec()==QDialog::Rejected) return Failed;
 
-  return OK;
-}
-
-
-KMBounceCommand::KMBounceCommand( QWidget *parent,
-  KMMessage *msg )
-  : KMCommand( parent, msg )
-{
-}
-
-KMCommand::Result KMBounceCommand::execute()
-{
-  KMMessage *msg = retrievedMessage();
-  KMMessage *newMsg = msg->createBounce( TRUE /* with UI */);
-  if (newMsg)
-    kmkernel->msgSender()->send(newMsg, kmkernel->msgSender()->sendImmediate());
+  KMMessage *newMsg = msg->createRedirect( dlg.to() );
+  KMFilterAction::sendMDN( msg, KMime::MDN::Dispatched );
 
+  if ( !kmkernel->msgSender()->send( newMsg, dlg.sendImmediate() ) ) {
+    kdDebug(5006) << "KMRedirectCommand: could not redirect message (sending failed)" << endl;
+    return Failed; // error: couldn't send
+  }
   return OK;
 }
 
 
 KMPrintCommand::KMPrintCommand( QWidget *parent,
-  KMMessage *msg, bool htmlOverride, const QTextCodec* codec )
-  : KMCommand( parent, msg ), mHtmlOverride( htmlOverride ), mCodec( codec )
+  KMMessage *msg, bool htmlOverride, bool htmlLoadExtOverride,
+  bool useFixedFont, const QTextCodec *codec )
+  : KMCommand( parent, msg ), mHtmlOverride( htmlOverride ),
+    mHtmlLoadExtOverride( htmlLoadExtOverride ),
+    mUseFixedFont( useFixedFont ), mCodec( codec )
 {
 }
 
 KMCommand::Result KMPrintCommand::execute()
 {
   KMReaderWin printWin( 0, 0, 0 );
-  printWin.setPrinting(TRUE);
+  printWin.setPrinting( true );
   printWin.readConfig();
   printWin.setHtmlOverride( mHtmlOverride );
+  printWin.setHtmlLoadExtOverride( mHtmlLoadExtOverride );
+  printWin.setUseFixedFont( mUseFixedFont );
   printWin.setOverrideCodec( mCodec );
-  printWin.setMsg(retrievedMessage(), TRUE);
+  printWin.setMsg( retrievedMessage(), true );
   printWin.printMsg();
 
   return OK;
@@ -1423,7 +1450,7 @@ void KMMetaFilterActionCommand::start()
   scheduler->setAutoDestruct( true );
 
   int contentX, contentY;
-  KMHeaderItem *nextItem = mHeaders->prepareMove( &contentX, &contentY );
+  HeaderItem *nextItem = mHeaders->prepareMove( &contentX, &contentY );
   QPtrList<KMMsgBase> msgList = *mHeaders->selectedMsgs(true);
   mHeaders->finalizeMove( nextItem, contentX, contentY );
 
@@ -1435,11 +1462,33 @@ void KMMetaFilterActionCommand::start()
   *mHeaders->selectedMsgs(), mFilter);
   filterCommand->start();
   int contentX, contentY;
-  KMHeaderItem *item = mHeaders->prepareMove( &contentX, &contentY );
+  HeaderItem *item = mHeaders->prepareMove( &contentX, &contentY );
   mHeaders->finalizeMove( item, contentX, contentY );
 #endif
 }
 
+FolderShortcutCommand::FolderShortcutCommand( KMMainWidget *mainwidget,
+                                              KMFolder *folder )
+    : mMainWidget( mainwidget ), mFolder( folder ), mAction( 0 )
+{
+}
+
+
+FolderShortcutCommand::~FolderShortcutCommand()
+{
+  if ( mAction ) mAction->unplugAll();
+  delete mAction;
+}
+
+void FolderShortcutCommand::start()
+{
+  mMainWidget->slotSelectFolder( mFolder );
+}
+
+void FolderShortcutCommand::setAction( KAction* action )
+{
+  mAction = action;
+}
 
 KMMailingListFilterCommand::KMMailingListFilterCommand( QWidget *parent,
                                                         KMMessage *msg )
@@ -1574,11 +1623,13 @@ KMCopyCommand::KMCopyCommand( KMFolder* destFolder,
                               const QPtrList<KMMsgBase> &msgList )
 :mDestFolder( destFolder ), mMsgList( msgList )
 {
+  setDeletesItself( true );
 }
 
 KMCopyCommand::KMCopyCommand( KMFolder* destFolder, KMMessage * msg )
   :mDestFolder( destFolder )
 {
+  setDeletesItself( true );
   mMsgList.append( &msg->toMsgBase() );
 }
 
@@ -1589,12 +1640,17 @@ KMCommand::Result KMCopyCommand::execute()
   int idx = -1;
   bool isMessage;
   QPtrList<KMMessage> list;
+  QPtrList<KMMessage> localList;
 
   if (mDestFolder && mDestFolder->open() != 0)
+  {
+    deleteLater();
     return Failed;
+  }
 
   KCursorSaver busy(KBusyPtr::busy());
 
+  mWaitingForMsgs.clear();
   for (msgBase = mMsgList.first(); msgBase; msgBase = mMsgList.next() )
   {
     KMFolder *srcFolder = msgBase->parent();
@@ -1613,6 +1669,7 @@ KMCommand::Result KMCopyCommand::execute()
         (static_cast<KMFolderImap*>(srcFolder->storage())->account() ==
          static_cast<KMFolderImap*>(mDestFolder->storage())->account()))
     {
+      // imap => imap with same account
       list.append(msg);
     } else {
       newMsg = new KMMessage;
@@ -1625,21 +1682,21 @@ KMCommand::Result KMCopyCommand::execute()
 
       if (srcFolder && !newMsg->isComplete())
       {
+        // imap => others
+        mWaitingForMsgs.append( msg->getMsgSerNum() );
+        disconnect(mDestFolder, SIGNAL(msgAdded(KMFolder*, Q_UINT32)),
+            this, SLOT(slotMsgAdded(KMFolder*, Q_UINT32)));
+        connect(mDestFolder, SIGNAL(msgAdded(KMFolder*, Q_UINT32)),
+            this, SLOT(slotMsgAdded(KMFolder*, Q_UINT32)));
         newMsg->setParent(msg->parent());
         FolderJob *job = srcFolder->createJob(newMsg);
         job->setCancellable( false );
         connect(job, SIGNAL(messageRetrieved(KMMessage*)),
                 mDestFolder, SLOT(reallyAddCopyOfMsg(KMMessage*)));
-        // msg musn't be deleted
-        newMsg->setTransferInProgress(true);
         job->start();
       } else {
-        int rc, index;
-        mDestFolder->open();
-        rc = mDestFolder->addMsg(newMsg, &index);
-        if (rc == 0 && index != -1)
-          mDestFolder->unGetMsg( mDestFolder->count() - 1 );
-        mDestFolder->close();
+        // local => others
+        localList.append(newMsg);
       }
     }
 
@@ -1650,7 +1707,19 @@ KMCommand::Result KMCopyCommand::execute()
     }
 
   } // end for
-  mDestFolder->close();
+
+  if (!localList.isEmpty())
+  {
+    QValueList<int> index;
+    mDestFolder->addMsg( localList, index );
+    for ( QValueListIterator<int> it = index.begin(); it != index.end(); ++it ) {
+      mDestFolder->unGetMsg( *it );
+    }
+  }
+
+  // only close the folder if we're done
+  if ( mWaitingForMsgs.isEmpty() )
+    mDestFolder->close();
 
 //TODO: Get rid of the other cases just use this one for all types of folder
 //TODO: requires adding copyMsg and getFolder methods to KMFolder.h
@@ -1663,9 +1732,21 @@ KMCommand::Result KMCopyCommand::execute()
     imapDestFolder->getFolder();
   }
 
+  if ( mWaitingForMsgs.isEmpty() )
+    deleteLater();
   return OK;
 }
 
+void KMCopyCommand::slotMsgAdded( KMFolder*, Q_UINT32 serNum )
+{
+  mWaitingForMsgs.remove( serNum );
+  if ( mWaitingForMsgs.isEmpty() )
+  {
+    mDestFolder->close();
+    deleteLater();
+  }
+}
+
 
 KMMoveCommand::KMMoveCommand( KMFolder* destFolder,
                               const QPtrList<KMMsgBase> &msgList)
@@ -1714,7 +1795,7 @@ KMCommand::Result KMMoveCommand::execute()
      ProgressManager::createProgressItem (
          "move"+ProgressManager::getUniqueID(),
          mDestFolder ? i18n( "Moving messages" ) : i18n( "Deleting messages" ) );
-  connect( mProgressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
+  connect( mProgressItem, SIGNAL( progressItemCanceled( KPIM::ProgressItem* ) ),
            this, SLOT( slotMoveCanceled() ) );
 
   KMMessage *msg;
@@ -1801,7 +1882,7 @@ KMCommand::Result KMMoveCommand::execute()
     }
   }
   if (!list.isEmpty() && mDestFolder) {
-       mDestFolder->moveMsg(list, &index);
+    mDestFolder->moveMsg(list, &index);
   } else {
     FolderToMessageListMap::Iterator it;
     for ( it = folderDeleteList.begin(); it != folderDeleteList.end(); ++it ) {
@@ -1818,7 +1899,8 @@ KMCommand::Result KMMoveCommand::execute()
         completeMove( OK );
       }
     }
-    if ( !mDestFolder ) {
+    if ( !mDestFolder || mLostBoys.isEmpty() ) {
+      // normal local move
       completeMove( OK );
     }
   }
@@ -1826,7 +1908,7 @@ KMCommand::Result KMMoveCommand::execute()
   return OK;
 }
 
-void KMMoveCommand::slotImapFolderCompleted(KMFolderImap *, bool success)
+void KMMoveCommand::slotImapFolderCompleted(KMFolderImap*, bool success)
 {
   if ( success ) {
     // the folder was checked successfully but we were still called, so check
@@ -1848,9 +1930,9 @@ void KMMoveCommand::slotImapFolderCompleted(KMFolderImap *, bool success)
 
 void KMMoveCommand::slotMsgAddedToDestFolder(KMFolder *folder, Q_UINT32 serNum)
 {
-  if (folder != mDestFolder || !mLostBoys.contains( serNum ) ) {
-    kdDebug(5006) << "KMMoveCommand::msgAddedToDestFolder different "
-                     "folder or invalid serial number." << endl;
+  if (folder != mDestFolder || mLostBoys.find( serNum ) == mLostBoys.end() ) {
+    //kdDebug(5006) << "KMMoveCommand::msgAddedToDestFolder different "
+    //                 "folder or invalid serial number." << endl;
     return;
   }
   mLostBoys.remove(serNum);
@@ -1858,8 +1940,8 @@ void KMMoveCommand::slotMsgAddedToDestFolder(KMFolder *folder, Q_UINT32 serNum)
     // we are done. All messages transferred to the host succesfully
     if (mDestFolder && mDestFolder->folderType() != KMFolderTypeImap) {
       mDestFolder->sync();
+      completeMove( OK ); // imap ones will be completed via folderComplete
     }
-    completeMove( OK );
   } else {
     mProgressItem->incCompletedItems();
     mProgressItem->updateProgress();
@@ -1870,8 +1952,16 @@ void KMMoveCommand::completeMove( Result result )
 {
   if ( mDestFolder )
     mDestFolder->close();
+  while ( !mOpenedFolders.empty() ) {
+    KMFolder *folder = mOpenedFolders.back();
+    mOpenedFolders.pop_back();
+    folder->close();
+  }
   if ( mProgressItem )
+  {
     mProgressItem->setComplete();
+    mProgressItem = 0;
+  }
   setResult( result );
   emit completed( this );
   deleteLater();
@@ -1887,11 +1977,15 @@ KMDeleteMsgCommand::KMDeleteMsgCommand( KMFolder* srcFolder,
   const QPtrList<KMMsgBase> &msgList )
 :KMMoveCommand( findTrashFolder( srcFolder ), msgList)
 {
+  srcFolder->open();
+  mOpenedFolders.push_back( srcFolder );
 }
 
 KMDeleteMsgCommand::KMDeleteMsgCommand( KMFolder* srcFolder, KMMessage * msg )
 :KMMoveCommand( findTrashFolder( srcFolder ), msg)
 {
+  srcFolder->open();
+  mOpenedFolders.push_back( srcFolder );
 }
 
 KMDeleteMsgCommand::KMDeleteMsgCommand( Q_UINT32 sernum )
@@ -1901,6 +1995,8 @@ KMDeleteMsgCommand::KMDeleteMsgCommand( Q_UINT32 sernum )
   int idx;
   kmkernel->msgDict()->getLocation( sernum, &srcFolder, &idx );
   KMMsgBase *msg = srcFolder->getMsgBase( idx );
+  srcFolder->open();
+  mOpenedFolders.push_back( srcFolder );
   addMsg( msg );
   setDestFolder( findTrashFolder( srcFolder ) );
 }
@@ -2104,6 +2200,8 @@ void KMSaveAttachmentsCommand::slotSaveAll()
     }
   }
 
+  QMap< QString, int > renameNumbering;
+
   Result globalResult = OK;
   int unnamedAtmCount = 0;
   for ( PartNodeMessageMap::const_iterator it = mAttachmentMap.begin();
@@ -2128,11 +2226,39 @@ void KMSaveAttachmentsCommand::slotSaveAll()
     }
 
     if ( !curUrl.isEmpty() ) {
+
+     // Rename the file if we have already saved one with the same name:
+     // try appending a number before extension (e.g. "pic.jpg" => "pic_2.jpg")
+     QString origFile = curUrl.fileName();
+     QString file = origFile;
+
+     while ( renameNumbering.contains(file) ) {
+       file = origFile;
+       int num = renameNumbering[file] + 1;
+       int dotIdx = file.findRev('.');
+       file = file.insert( (dotIdx>=0) ? dotIdx : file.length(), QString("_") + QString::number(num) );
+     }
+     curUrl.setFileName(file);
+
+     // Increment the counter for both the old and the new filename
+     if ( !renameNumbering.contains(origFile))
+         renameNumbering[origFile] = 1;
+     else
+         renameNumbering[origFile]++;
+
+     if ( file != origFile ) {
+        if ( !renameNumbering.contains(file))
+            renameNumbering[file] = 1;
+        else
+            renameNumbering[file]++;
+     }
+
+
       if ( KIO::NetAccess::exists( curUrl, false, parentWidget() ) ) {
         if ( KMessageBox::warningContinueCancel( parentWidget(),
               i18n( "A file named %1 already exists. Do you want to overwrite it?" )
               .arg( curUrl.fileName() ),
-              i18n( "File Already Exists" ), i18n("Overwrite") ) == KMessageBox::Cancel) {
+              i18n( "File Already Exists" ), i18n("&Overwrite") ) == KMessageBox::Cancel) {
           continue;
         }
       }
@@ -2464,28 +2590,29 @@ KMIMChatCommand::KMIMChatCommand( const KURL &url, KMMessage *msg )
 KMCommand::Result KMIMChatCommand::execute()
 {
   kdDebug( 5006 ) << k_funcinfo << " URL is: " << mUrl << endl;
+  QString addr = KMMessage::decodeMailtoUrl( mUrl.path() );
   // find UID for mail address
   KABC::AddressBook *addressBook = KABC::StdAddressBook::self();
-  KABC::AddresseeList addresses = addressBook->findByEmail( KPIM::getEmailAddr( mUrl.path() ) ) ;
+  KABC::AddresseeList addressees = addressBook->findByEmail( KPIM::getEmailAddress( addr ) ) ;
 
   // start chat
-  if( addresses.count() == 1 ) {
-    kmkernel->imProxy()->chatWithContact( addresses[0].uid() );
+  if( addressees.count() == 1 ) {
+    kmkernel->imProxy()->chatWithContact( addressees[0].uid() );
     return OK;
   }
   else
   {
-    kdDebug( 5006 ) << "Didn't find exactly one addressee, couldn't tell who to chat to for that email address.  Count = " << addresses.count() << endl;
+    kdDebug( 5006 ) << "Didn't find exactly one addressee, couldn't tell who to chat to for that email address.  Count = " << addressees.count() << endl;
 
     QString apology;
-    if ( addresses.isEmpty() )
+    if ( addressees.isEmpty() )
       apology = i18n( "There is no Address Book entry for this email address. Add them to the Address Book and then add instant messaging addresses using your preferred messaging client." );
     else
     {
       apology = i18n( "More than one Address Book entry uses this email address:\n %1\n it is not possible to determine who to chat with." );
       QStringList nameList;
-      KABC::AddresseeList::const_iterator it = addresses.begin();
-      KABC::AddresseeList::const_iterator end = addresses.end();
+      KABC::AddresseeList::const_iterator it = addressees.begin();
+      KABC::AddresseeList::const_iterator end = addressees.end();
       for ( ; it != end; ++it )
       {
           nameList.append( (*it).realName() );
@@ -2498,3 +2625,185 @@ KMCommand::Result KMIMChatCommand::execute()
     return Failed;
   }
 }
+
+KMHandleAttachmentCommand::KMHandleAttachmentCommand( partNode* node,
+     KMMessage* msg, int atmId, const QString& atmName,
+     AttachmentAction action, KService::Ptr offer )
+: mNode( node ), mMsg( msg ), mAtmId( atmId ), mAtmName( atmName ),
+  mAction( action ), mOffer( offer )
+{
+}
+
+void KMHandleAttachmentCommand::slotStart()
+{
+  if ( !mNode->msgPart().isComplete() )
+  {
+    // load the part
+    KMLoadPartsCommand *command = new KMLoadPartsCommand( mNode, mMsg );
+    connect( command, SIGNAL( partsRetrieved() ),
+        this, SLOT( slotPartComplete() ) );
+    command->start();
+  } else
+  {
+    execute();
+  }
+}
+
+void KMHandleAttachmentCommand::slotPartComplete()
+{
+  execute();
+}
+
+KMCommand::Result KMHandleAttachmentCommand::execute()
+{
+  switch( mAction )
+  {
+    case Open:
+      atmOpen();
+      break;
+    case OpenWith:
+      atmOpenWith();
+      break;
+    case View:
+      atmView();
+      break;
+    case Save:
+      atmSave();
+      break;
+    case Properties:
+      atmProperties();
+      break;
+    default:
+      kdDebug(5006) << "unknown action " << mAction << endl;
+      break;
+  }
+  setResult( OK );
+  emit completed( this );
+  deleteLater();
+  return OK;
+}
+
+QString KMHandleAttachmentCommand::createAtmFileLink() const
+{
+  QFileInfo atmFileInfo( mAtmName );
+
+  if ( atmFileInfo.size() == 0 )
+  {
+    kdDebug(5006) << k_funcinfo << "rewriting attachment" << endl;
+    // there is something wrong so write the file again
+    QByteArray data = mNode->msgPart().bodyDecodedBinary();
+    size_t size = data.size();
+    if ( mNode->msgPart().type() == DwMime::kTypeText && size) {
+      // convert CRLF to LF before writing text attachments to disk
+      size = KMFolder::crlf2lf( data.data(), size );
+    }
+    KPIM::kBytesToFile( data.data(), size, mAtmName, false, false, false );
+  }
+
+  KTempFile *linkFile = new KTempFile( locateLocal("tmp", atmFileInfo.fileName() +"_["),
+                          "]."+ atmFileInfo.extension() );
+
+  linkFile->setAutoDelete(true);
+  QString linkName = linkFile->name();
+  delete linkFile;
+
+  if ( ::link(QFile::encodeName( mAtmName ), QFile::encodeName( linkName )) == 0 ) {
+    return linkName; // success
+  }
+  return QString::null;
+}
+
+KService::Ptr KMHandleAttachmentCommand::getServiceOffer()
+{
+  KMMessagePart& msgPart = mNode->msgPart();
+  const QString contentTypeStr =
+    ( msgPart.typeStr() + '/' + msgPart.subtypeStr() ).lower();
+
+  if ( contentTypeStr == "text/x-vcard" ) {
+    atmView();
+    return 0;
+  }
+  // determine the MIME type of the attachment
+  KMimeType::Ptr mimetype;
+  // prefer the value of the Content-Type header
+  mimetype = KMimeType::mimeType( contentTypeStr );
+  if ( mimetype->name() == "application/octet-stream" ) {
+    // consider the filename if Content-Type is application/octet-stream
+    mimetype = KMimeType::findByPath( mAtmName, 0, true /* no disk access */ );
+  }
+  if ( ( mimetype->name() == "application/octet-stream" )
+       && msgPart.isComplete() ) {
+    // consider the attachment's contents if neither the Content-Type header
+    // nor the filename give us a clue
+    mimetype = KMimeType::findByFileContent( mAtmName );
+  }
+  return KServiceTypeProfile::preferredService( mimetype->name(), "Application" );
+}
+
+void KMHandleAttachmentCommand::atmOpen()
+{
+  if ( !mOffer )
+    mOffer = getServiceOffer();
+  if ( !mOffer )
+    return;
+
+  KURL::List lst;
+  KURL url;
+  bool autoDelete = true;
+  QString fname = createAtmFileLink();
+
+  if ( fname.isNull() ) {
+    autoDelete = false;
+    fname = mAtmName;
+  }
+
+  url.setPath( fname );
+  lst.append( url );
+  if ( (KRun::run( *mOffer, lst, autoDelete ) <= 0) && autoDelete ) {
+      QFile::remove(url.path());
+  }
+}
+
+void KMHandleAttachmentCommand::atmOpenWith()
+{
+  KURL::List lst;
+  KURL url;
+  bool autoDelete = true;
+  QString fname = createAtmFileLink();
+
+  if ( fname.isNull() ) {
+    autoDelete = false;
+    fname = mAtmName;
+  }
+
+  url.setPath( fname );
+  lst.append( url );
+  if ( (! KRun::displayOpenWithDialog(lst, autoDelete)) && autoDelete ) {
+    QFile::remove( url.path() );
+  }
+}
+
+void KMHandleAttachmentCommand::atmView()
+{
+  // we do not handle this ourself
+  emit showAttachment( mAtmId, mAtmName );
+}
+
+void KMHandleAttachmentCommand::atmSave()
+{
+  QPtrList<partNode> parts;
+  parts.append( mNode );
+  // save, do not leave encoded
+  KMSaveAttachmentsCommand *command =
+    new KMSaveAttachmentsCommand( 0, parts, mMsg, false );
+  command->start();
+}
+
+void KMHandleAttachmentCommand::atmProperties()
+{
+  KMMsgPartDialogCompat dlg( 0, true );
+  KMMessagePart& msgPart = mNode->msgPart();
+  dlg.setMsgPart( &msgPart );
+  dlg.exec();
+}
+
