@@ -51,9 +51,9 @@ using KPIM::BroadcastStatus;
 #include "progressmanager.h"
 
 using KMail::CachedImapJob;
+#include "imapaccountbase.h"
 using KMail::ImapAccountBase;
 #include "listjob.h"
-using KMail::ListJob;
 
 #include <kapplication.h>
 #include <kmessagebox.h>
@@ -67,9 +67,27 @@ using KMail::ListJob;
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qvaluelist.h>
+#include "annotationjobs.h"
+using namespace KMail;
+#include <globalsettings.h>
 
 #define UIDCACHE_VERSION 1
 
+static QString incidencesForToString( KMFolderCachedImap::IncidencesFor r ) {
+  switch (r) {
+  case KMFolderCachedImap::IncForNobody: return "nobody";
+  case KMFolderCachedImap::IncForAdmins: return "admins";
+  case KMFolderCachedImap::IncForReaders: return "readers";
+  }
+  return QString::null; // can't happen
+}
+
+static KMFolderCachedImap::IncidencesFor incidencesForFromString( const QString& str ) {
+  if ( str == "nobody" ) return KMFolderCachedImap::IncForNobody;
+  if ( str == "admins" ) return KMFolderCachedImap::IncForAdmins;
+  if ( str == "readers" ) return KMFolderCachedImap::IncForReaders;
+  return KMFolderCachedImap::IncForAdmins; // by default
+}
 
 DImapTroubleShootDialog::DImapTroubleShootDialog( QWidget* parent,
                                                   const char* name )
@@ -120,12 +138,16 @@ void DImapTroubleShootDialog::slotRebuildIndex()
 KMFolderCachedImap::KMFolderCachedImap( KMFolder* folder, const char* aName )
   : KMFolderMaildir( folder, aName ),
     mSyncState( SYNC_STATE_INITIAL ), mContentState( imapNoInformation ),
-    mSubfolderState( imapNoInformation ), mIsSelected( false ),
+    mSubfolderState( imapNoInformation ),
+    mIncidencesFor( IncForAdmins ),
+    mIsSelected( false ),
     mCheckFlags( true ), mAccount( NULL ), uidMapDirty( true ),
     uidWriteTimer( -1 ), mLastUid( 0 ), mTentativeHighestUid( 0 ),
-    mUserRights( 0 ), mFolderRemoved( false ),
+    mUserRights( 0 ), mSilentUpload( false ),
+    mFolderRemoved( false ),
     /*mHoldSyncs( false ),*/ mRecurse( true ),
-    mContentsTypeChanged( false ), mStatusChangedLocally( false )
+    mStatusChangedLocally( false ), mAnnotationFolderTypeChanged( false ),
+    mIncidencesForChanged( false )
 {
   setUidValidity("");
   readUidCache();
@@ -136,14 +158,7 @@ KMFolderCachedImap::KMFolderCachedImap( KMFolder* folder, const char* aName )
 KMFolderCachedImap::~KMFolderCachedImap()
 {
   if( !mFolderRemoved ) {
-    // Only write configuration when the folder haven't been deleted
-    KConfig* config = KMKernel::config();
-    KConfigGroupSaver saver( config, "Folder-" + folder()->idString() );
-    config->writeEntry( "ImapPath", mImapPath );
-    config->writeEntry( "NoContent", mNoContent );
-    config->writeEntry( "ReadOnly", mReadOnly );
-    config->writeEntry( "StatusChangedLocally", mStatusChangedLocally );
-
+    writeConfig();
     writeUidCache();
   }
 
@@ -173,10 +188,48 @@ void KMFolderCachedImap::readConfig()
   mNoContent = config->readBoolEntry( "NoContent", false );
   mReadOnly = config->readBoolEntry( "ReadOnly", false );
 
+  if ( mAnnotationFolderType != "FROMSERVER" ) {
+    mAnnotationFolderType = config->readEntry( "Annotation-FolderType" );
+    // if there is an annotation, it has to XML
+    if ( !mAnnotationFolderType.isEmpty() )
+      kmkernel->iCalIface().setStorageFormat( folder(), KMailICalIfaceImpl::StorageXML );
+//    kdDebug(5006) << ( mImapPath.isEmpty() ? label() : mImapPath )
+//                  << " readConfig: mAnnotationFolderType=" << mAnnotationFolderType << endl;
+  }
+  mIncidencesFor = incidencesForFromString( config->readEntry( "IncidencesFor" ) );
+//  kdDebug(5006) << ( mImapPath.isEmpty() ? label() : mImapPath )
+//                << " readConfig: mIncidencesFor=" << mIncidencesFor << endl;
+
   KMFolderMaildir::readConfig();
-  mContentsTypeChanged = false;
+
   mStatusChangedLocally =
     config->readBoolEntry( "StatusChangedLocally", false );
+
+  mAnnotationFolderTypeChanged = config->readBoolEntry( "AnnotationFolderTypeChanged", false );
+  mIncidencesForChanged = config->readBoolEntry( "IncidencesForChanged", false );
+}
+
+void KMFolderCachedImap::writeConfig()
+{
+  KConfigGroup configGroup( KMKernel::config(), "Folder-" + folder()->idString() );
+  configGroup.writeEntry( "ImapPath", mImapPath );
+  configGroup.writeEntry( "NoContent", mNoContent );
+  configGroup.writeEntry( "ReadOnly", mReadOnly );
+  configGroup.writeEntry( "StatusChangedLocally", mStatusChangedLocally );
+  writeAnnotationConfig();
+  KMFolderMaildir::writeConfig();
+}
+
+void KMFolderCachedImap::writeAnnotationConfig()
+{
+  KConfigGroup configGroup( KMKernel::config(), "Folder-" + folder()->idString() );
+  if ( !folder()->noContent() )
+  {
+    configGroup.writeEntry( "AnnotationFolderTypeChanged", mAnnotationFolderTypeChanged );
+    configGroup.writeEntry( "Annotation-FolderType", mAnnotationFolderType );
+    configGroup.writeEntry( "IncidencesForChanged", mIncidencesForChanged );
+    configGroup.writeEntry( "IncidencesFor", incidencesForToString( mIncidencesFor ) );
+  }
 }
 
 void KMFolderCachedImap::remove()
@@ -295,12 +348,10 @@ int KMFolderCachedImap::addMsgInternal( KMMessage* msg, bool newMail,
 /* Reimplemented from KMFolderMaildir */
 int KMFolderCachedImap::addMsg(KMMessage* msg, int* index_return)
 {
-  // Strip the IMAP UID
-  msg->removeHeaderField( "X-UID" );
-  msg->setUID( 0 );
-
+  if ( !canAddMsgNow( msg, index_return ) ) return 0;
   // Add it to storage
-  return addMsgInternal( msg, false, index_return );
+  int rc = KMFolderMaildir::addMsgInternal(msg, index_return, true /*stripUID*/);
+  return rc;
 }
 
 
@@ -328,7 +379,9 @@ bool KMFolderCachedImap::canRemoveFolder() const {
 int KMFolderCachedImap::rename( const QString& aName,
                                 KMFolderDir* /*aParent*/ )
 {
-  if ( aName == name() )
+  QString oldName = mAccount->renamedFolder( imapPath() );
+  if ( oldName.isEmpty() ) oldName = name();
+  if ( aName == oldName )
     // Stupid user trying to rename it to it's old name :)
     return 0;
 
@@ -339,9 +392,17 @@ int KMFolderCachedImap::rename( const QString& aName,
   }
 
   // Make the change appear to the user with setLabel, but we'll do the change
-  // on the server during the next sync.
-  mAccount->addRenamedFolder( imapPath(), folder()->label(), aName );
+  // on the server during the next sync. The name() is the name at the time of
+  // the last sync. Only rename if the new one is different. If it's the same, 
+  // don't rename, but also make sure the rename is reset, in the case of
+  // A -> B -> A renames.
+  if ( name() != aName )
+    mAccount->addRenamedFolder( imapPath(), folder()->label(), aName );
+  else
+    mAccount->removeRenamedFolder( imapPath() );
+
   folder()->setLabel( aName );
+  emit nameChanged(); // for kmailicalifaceimpl
 
   return 0;
 }
@@ -424,9 +485,9 @@ void KMFolderCachedImap::slotTroubleshoot()
       return;
     }
     QString str = i18n("Are you sure you want to refresh the IMAP cache of "
-                       "the folder %1 and all it's subfolders?\nThis will "
+                       "the folder %1 and all its subfolders?\nThis will "
                        "remove all changes you have done locally to your "
-                       "folders").arg( label() );
+                       "folders.").arg( label() );
     QString s1 = i18n("Refresh IMAP Cache");
     QString s2 = i18n("&Refresh");
     if( KMessageBox::warningContinueCancel( 0, str, s1, s2 ) ==
@@ -475,6 +536,7 @@ QString KMFolderCachedImap::state2String( int state ) const
 {
   switch( state ) {
   case SYNC_STATE_INITIAL:           return "SYNC_STATE_INITIAL";
+  case SYNC_STATE_TEST_ANNOTATIONS:  return "SYNC_STATE_TEST_ANNOTATIONS";
   case SYNC_STATE_PUT_MESSAGES:      return "SYNC_STATE_PUT_MESSAGES";
   case SYNC_STATE_UPLOAD_FLAGS:      return "SYNC_STATE_UPLOAD_FLAGS";
   case SYNC_STATE_CREATE_SUBFOLDERS: return "SYNC_STATE_CREATE_SUBFOLDERS";
@@ -487,6 +549,8 @@ QString KMFolderCachedImap::state2String( int state ) const
   case SYNC_STATE_EXPUNGE_MESSAGES:  return "SYNC_STATE_EXPUNGE_MESSAGES";
   case SYNC_STATE_HANDLE_INBOX:      return "SYNC_STATE_HANDLE_INBOX";
   case SYNC_STATE_GET_USERRIGHTS:    return "SYNC_STATE_GET_USERRIGHTS";
+  case SYNC_STATE_GET_ANNOTATIONS:   return "SYNC_STATE_GET_ANNOTATIONS";
+  case SYNC_STATE_SET_ANNOTATIONS:   return "SYNC_STATE_SET_ANNOTATIONS";
   case SYNC_STATE_GET_ACLS:          return "SYNC_STATE_GET_ACLS";
   case SYNC_STATE_SET_ACLS:          return "SYNC_STATE_SET_ACLS";
   case SYNC_STATE_FIND_SUBFOLDERS:   return "SYNC_STATE_FIND_SUBFOLDERS";
@@ -515,8 +579,10 @@ QString KMFolderCachedImap::state2String( int state ) const
    delete_messages 10
    expunge_messages 5
    get_messages variable (remaining-5) i.e. minimum 15.
+   set_annotations 0 (rare)
+   get_annotations 2
    set_acls 0 (rare)
-   get_acls 5
+   get_acls 3
 
   noContent folders have only a few of the above steps
   (permissions, and all subfolder stuff), so its steps should be given more span
@@ -558,6 +624,7 @@ void KMFolderCachedImap::serverSyncInternal()
       emit folderComplete(this, FALSE);
       break;
     } else if ( cs == ImapAccountBase::Connecting ) {
+      mAccount->setAnnotationCheckPassed( false );
       // kdDebug(5006) << "makeConnection said Connecting, waiting for signal." << endl;
       newState( mProgress, i18n("Connecting to %1").arg( mAccount->host() ) );
       // We'll wait for the connectionResult signal from the account.
@@ -571,9 +638,12 @@ void KMFolderCachedImap::serverSyncInternal()
       // Fall through to next state
     }
   }
+  
 
   case SYNC_STATE_GET_USERRIGHTS:
-    mSyncState = SYNC_STATE_RENAME_FOLDER;
+    //kdDebug(5006) << "===== Syncing " << ( mImapPath.isEmpty() ? label() : mImapPath ) << endl;
+
+    mSyncState = SYNC_STATE_TEST_ANNOTATIONS;
 
     if( !noContent() && mAccount->hasACLSupport() ) {
       // Check the user's own rights. We do this every time in case they changed.
@@ -584,12 +654,40 @@ void KMFolderCachedImap::serverSyncInternal()
       break;
     }
 
+  #define KOLAB_FOLDERTEST "/vendor/kolab/folder-test"
+  case SYNC_STATE_TEST_ANNOTATIONS:
+    mSyncState = SYNC_STATE_RENAME_FOLDER;
+    // The first folder with user rights to write annotations
+    if( !mAccount->annotationCheckPassed() &&
+         ( mUserRights <= 0 || ( mUserRights & ACLJobs::Administer ) )  
+      ){
+      newState( mProgress, i18n("Checking annotation support"));
+    
+      KURL url = mAccount->getUrl();
+      url.setPath( imapPath() );
+      KMail::AnnotationList annotations; // to be set
+      
+      KMail::AnnotationAttribute attr( KOLAB_FOLDERTEST, "value.shared", "true" );
+      annotations.append( attr );
+      
+      kdDebug(5006) << "Setting test attribute to "<< url << endl;
+      KIO::Job* job = AnnotationJobs::multiSetAnnotation( mAccount->slave(), 
+          url, annotations );
+      ImapAccountBase::jobData jd( url.url(), folder() );
+      jd.cancellable = true; // we can always do so later
+      mAccount->insertJob(job, jd);
+       connect(job, SIGNAL(result(KIO::Job *)),
+              SLOT(slotTestAnnotationResult(KIO::Job *)));
+      break;
+    }
+    
   case SYNC_STATE_RENAME_FOLDER:
   {
     mSyncState = SYNC_STATE_CHECK_UIDVALIDITY;
     // Returns the new name if the folder was renamed, empty otherwise.
+    bool isResourceFolder = kmkernel->iCalIface().isStandardResourceFolder( folder() );
     QString newName = mAccount->renamedFolder( imapPath() );
-    if ( !newName.isEmpty() ) {
+    if ( !newName.isEmpty() && !folder()->isSystemFolder() && !isResourceFolder ) {
       newState( mProgress, i18n("Renaming folder") );
       CachedImapJob *job = new CachedImapJob( newName, CachedImapJob::tRenameFolder, this );
       connect( job, SIGNAL( result(KMail::FolderJob *) ), this, SLOT( slotIncreaseProgress() ) );
@@ -632,7 +730,7 @@ void KMFolderCachedImap::serverSyncInternal()
            uploadFlags();
            break;
          } else {
-           kdDebug(5006) << "Skipping flags upload, folder unchanged: " << label() << endl;
+           //kdDebug(5006) << "Skipping flags upload, folder unchanged: " << label() << endl;
          }
        }
     }
@@ -740,13 +838,88 @@ void KMFolderCachedImap::serverSyncInternal()
     // Wrap up the 'download emails' stage. We always end up at 95 here.
     mProgress = 95;
 
-    // Continue with the ACLs
+    mSyncState = SYNC_STATE_GET_ANNOTATIONS;
+
+  case SYNC_STATE_GET_ANNOTATIONS:
+#define KOLAB_FOLDERTYPE "/vendor/kolab/folder-type"
+#define KOLAB_INCIDENCESFOR "/vendor/kolab/incidences-for"
+//#define KOLAB_FOLDERTYPE "/comment"  //for testing, while cyrus-imap doesn't support /vendor/*
+    mSyncState = SYNC_STATE_SET_ANNOTATIONS;
+
+    if ( !noContent() ) {
+      // for a folder we didn't create ourselves: get annotation from server
+      if ( mAnnotationFolderType == "FROMSERVER" )
+        mAnnotationFolderType = QString::null;
+      else
+        updateAnnotationFolderType();
+    }
+
+    // First retrieve the annotation, so that we know we have to set it if it's not set.
+    // On the other hand, if the user changed the contentstype, there's no need to get first.
+    if ( !noContent() && mAccount->hasAnnotationSupport() && kmkernel->iCalIface().isEnabled() ) {
+      QStringList annotations; // list of annotations to be fetched
+      if ( !mAnnotationFolderTypeChanged || mAnnotationFolderType.isEmpty() )
+        annotations << KOLAB_FOLDERTYPE;
+      if ( !mIncidencesForChanged )
+        annotations << KOLAB_INCIDENCESFOR;
+      if ( !annotations.isEmpty() ) {
+        newState( mProgress, i18n("Retrieving annotations"));
+        KURL url = mAccount->getUrl();
+        url.setPath( imapPath() );
+        AnnotationJobs::MultiGetAnnotationJob* job =
+          AnnotationJobs::multiGetAnnotation( mAccount->slave(), url, annotations );
+        ImapAccountBase::jobData jd( url.url(), folder() );
+        jd.cancellable = true;
+        mAccount->insertJob(job, jd);
+
+        connect( job, SIGNAL(annotationResult(const QString&, const QString&, bool)),
+                 SLOT(slotAnnotationResult(const QString&, const QString&, bool)) );
+        connect( job, SIGNAL(result(KIO::Job *)),
+                 SLOT(slotGetAnnotationResult(KIO::Job *)) );
+        break;
+      }
+    }
+
+  case SYNC_STATE_SET_ANNOTATIONS:
+
     mSyncState = SYNC_STATE_SET_ACLS;
+    if ( !noContent() && mAccount->hasAnnotationSupport() &&
+         ( mUserRights <= 0 || ( mUserRights & ACLJobs::Administer ) ) ) {
+      newState( mProgress, i18n("Setting annotations"));
+      KURL url = mAccount->getUrl();
+      url.setPath( imapPath() );
+      KMail::AnnotationList annotations; // to be set
+      if ( mAnnotationFolderTypeChanged && !mAnnotationFolderType.isEmpty() ) {
+        KMail::AnnotationAttribute attr( KOLAB_FOLDERTYPE, "value.shared", mAnnotationFolderType );
+        annotations.append( attr );
+        kdDebug(5006) << "Setting folder-type annotation for " << label() << " to " << mAnnotationFolderType << endl;
+      }
+      if ( mIncidencesForChanged ) {
+        const QString val = incidencesForToString( mIncidencesFor );
+        KMail::AnnotationAttribute attr( KOLAB_INCIDENCESFOR, "value.shared", val );
+        annotations.append( attr );
+        kdDebug(5006) << "Setting incidences-for annotation for " << label() << " to " << val << endl;
+      }
+      if ( !annotations.isEmpty() ) {
+        KIO::Job* job =
+          AnnotationJobs::multiSetAnnotation( mAccount->slave(), url, annotations );
+        ImapAccountBase::jobData jd( url.url(), folder() );
+        jd.cancellable = true; // we can always do so later
+        mAccount->insertJob(job, jd);
+
+        connect(job, SIGNAL(annotationChanged( const QString&, const QString&, const QString& ) ),
+                SLOT( slotAnnotationChanged( const QString&, const QString&, const QString& ) ));
+        connect(job, SIGNAL(result(KIO::Job *)),
+                SLOT(slotSetAnnotationResult(KIO::Job *)));
+        break;
+      }
+    }
 
   case SYNC_STATE_SET_ACLS:
     mSyncState = SYNC_STATE_GET_ACLS;
 
-    if( !noContent() && mAccount->hasACLSupport() ) {
+    if( !noContent() && mAccount->hasACLSupport() &&
+      ( mUserRights <= 0 || ( mUserRights & ACLJobs::Administer ) ) ) {
       bool hasChangedACLs = false;
       ACLList::ConstIterator it = mACLList.begin();
       for ( ; it != mACLList.end() && !hasChangedACLs; ++it ) {
@@ -802,11 +975,14 @@ void KMFolderCachedImap::serverSyncInternal()
           node = folder()->child()->next();
         }
       }
-    }
 
     // All done for this folder.
     mProgress = 100; // all done
     newState( mProgress, i18n("Synchronization done"));
+      KURL url = mAccount->getUrl();
+      url.setPath( imapPath() );
+      kmkernel->iCalIface().folderSynced( folder(), url );
+    }
 
     if ( !mRecurse ) // "check mail for this folder" only
       mSubfoldersForSync.clear();
@@ -831,10 +1007,11 @@ void KMFolderCachedImap::serverSyncInternal()
         connect( mCurrentSubfolder, SIGNAL( folderComplete(KMFolderCachedImap*, bool) ),
                  this, SLOT( slotSubFolderComplete(KMFolderCachedImap*, bool) ) );
 
-        // kdDebug(5006) << "Sync'ing subfolder " << mCurrentSubfolder->imapPath() << endl;
+        //kdDebug(5006) << "Sync'ing subfolder " << mCurrentSubfolder->imapPath() << endl;
         assert( !mCurrentSubfolder->imapPath().isEmpty() );
         mCurrentSubfolder->setAccount( account() );
-        mCurrentSubfolder->serverSync( mRecurse /*which is true*/ );
+        bool recurse = mCurrentSubfolder->noChildren() ? false : true;
+        mCurrentSubfolder->serverSync( recurse );
       }
     }
     break;
@@ -1132,10 +1309,15 @@ void KMFolderCachedImap::slotGetMessagesData(KIO::Job * job, const QByteArray &
       setUidValidity((*it).cdata.mid(a + 17, b - a - 17));
     }
     a = (*it).cdata.find("\r\nX-Access:");
-    if (a != -1) {
+    // Only trust X-Access (i.e. the imap select info) if we don't know mUserRights.
+    // The latter is more accurate (checked on every sync) whereas X-Access is only
+    // updated when selecting the folder again, which might not happen if using
+    // RMB / Check Mail in this folder. We don't need two (potentially conflicting)
+    // sources for the readonly setting, in any case.
+    if (a != -1 && mUserRights == -1 ) {
       int b = (*it).cdata.find("\r\n", a + 12);
-      QString access = (*it).cdata.mid(a + 12, b - a - 12);
-      mReadOnly = access == "Read only";
+      const QString access = (*it).cdata.mid(a + 12, b - a - 12);
+      setReadOnly( access == "Read only" );
     }
     (*it).cdata.remove(0, pos);
   }
@@ -1263,7 +1445,7 @@ bool KMFolderCachedImap::listDirectory(bool secondStep)
   ImapAccountBase::ListType type = ImapAccountBase::List;
   if ( mAccount->onlySubscribedFolders() )
     type = ImapAccountBase::ListSubscribed;
-  ListJob* job = new ListJob( this, mAccount, type, secondStep,
+  KMail::ListJob* job = new KMail::ListJob( this, mAccount, type, secondStep,
       false, mAccount->hasInbox() );
   connect( job, SIGNAL(receivedFolders(const QStringList&, const QStringList&,
           const QStringList&, const QStringList&, const ImapAccountBase::jobData&)),
@@ -1412,6 +1594,8 @@ void KMFolderCachedImap::listDirectory2() {
           f->close();
           f->setAccount(mAccount);
           kmkernel->dimapFolderMgr()->contentsChanged();
+          f->mAnnotationFolderType = "FROMSERVER";
+          //kdDebug(5006) << subfolderPath << ": mAnnotationFolderType set to FROMSERVER" << endl;
         } else {
           kdDebug(5006) << "can't create folder " << mSubfolderNames[i] <<endl;
         }
@@ -1508,12 +1692,21 @@ KMFolderCachedImap::slotReceivedUserRights( KMFolder* folder )
     if ( mUserRights == 0 ) // didn't work
       mUserRights = -1; // error code (used in folderdia)
     else
-      mReadOnly = ( mUserRights & KMail::ACLJobs::Insert ) == 0;
+      setReadOnly( ( mUserRights & KMail::ACLJobs::Insert ) == 0 );
     mProgress += 5;
     serverSyncInternal();
   }
 }
 
+void
+KMFolderCachedImap::setReadOnly( bool readOnly )
+{
+  if ( readOnly != mReadOnly ) {
+    mReadOnly = readOnly;
+    emit readOnlyChanged( folder() );
+  }
+}
+
 void
 KMFolderCachedImap::slotReceivedACL( KMFolder* folder, KIO::Job*, const KMail::ACLList& aclList )
 {
@@ -1542,6 +1735,8 @@ KMFolderCachedImap::slotMultiSetACLResult(KIO::Job *job)
     // Display error but don't abort the sync just for this
     // PENDING(dfaure) reconsider using handleJobError now that it offers continue/cancel
     job->showErrorDialog();
+  else
+    kmkernel->iCalIface().addFolderChange( folder(), KMailICalIfaceImpl::ACL );
 
   if (mAccount->slave()) mAccount->removeJob(job);
   serverSyncInternal();
@@ -1570,7 +1765,7 @@ void KMFolderCachedImap::resetSyncState()
   mSyncState = SYNC_STATE_INITIAL;
   close();
   // Don't use newState here, it would revert to mProgress (which is < current value when listing messages)
-  ProgressItem *progressItem = mAccount->mailCheckProgressItem();
+  KPIM::ProgressItem *progressItem = mAccount->mailCheckProgressItem();
   QString str = i18n("Aborted");
   if (progressItem)
      progressItem->setStatus( str );
@@ -1585,7 +1780,7 @@ void KMFolderCachedImap::slotIncreaseProgress()
 void KMFolderCachedImap::newState( int progress, const QString& syncStatus )
 {
   //kdDebug() << k_funcinfo << folder() << " " << mProgress << " " << syncStatus << endl;
-  ProgressItem *progressItem = mAccount->mailCheckProgressItem();
+  KPIM::ProgressItem *progressItem = mAccount->mailCheckProgressItem();
   if( progressItem )
     progressItem->setCompletedItems( progress );
   if ( !syncStatus.isEmpty() ) {
@@ -1626,12 +1821,187 @@ void KMFolderCachedImap::setImapPath(const QString &path)
   mImapPath = path;
 }
 
-void KMFolderCachedImap::setContentsType( KMail::FolderContentsType type )
+// mAnnotationFolderType is the annotation as known to the server (and stored in kmailrc)
+// It is updated from the folder contents type and whether it's a standard resource folder.
+// This happens during the syncing phase and during initFolder for a new folder.
+// Don't do it earlier, e.g. from setContentsType:
+// on startup, it's too early there to know if this is a standard resource folder.
+void KMFolderCachedImap::updateAnnotationFolderType()
 {
-  if ( type != mContentsType ) {
-    FolderStorage::setContentsType( type );
-    mContentsTypeChanged = true;
+  QString oldType = mAnnotationFolderType;
+  QString oldSubType;
+  int dot = oldType.find( '.' );
+  if ( dot != -1 ) {
+    oldType.truncate( dot );
+    oldSubType = mAnnotationFolderType.mid( dot + 1 );
   }
+
+  QString newType, newSubType;
+  // We want to store an annotation on the folder only if using the kolab storage.
+  if ( kmkernel->iCalIface().storageFormat( folder() ) == KMailICalIfaceImpl::StorageXML ) {
+    newType = KMailICalIfaceImpl::annotationForContentsType( mContentsType );
+    if ( kmkernel->iCalIface().isStandardResourceFolder( folder() ) )
+      newSubType = "default";
+    else
+      newSubType = oldSubType; // preserve unknown subtypes, like drafts etc. And preserve ".default" too.
+  }
+
+  //kdDebug(5006) << mImapPath << ": updateAnnotationFolderType: " << newType << " " << newSubType << endl;
+  if ( newType != oldType || newSubType != oldSubType ) {
+    mAnnotationFolderType = newType + ( newSubType.isEmpty() ? QString::null : "."+newSubType );
+    mAnnotationFolderTypeChanged = true; // force a "set annotation" on next sync
+    kdDebug(5006) << mImapPath << ": updateAnnotationFolderType: '" << mAnnotationFolderType << "', was (" << oldType << " " << oldSubType << ") => mAnnotationFolderTypeChanged set to TRUE" << endl;
+  }
+  // Ensure that further readConfig()s don't lose mAnnotationFolderType
+  writeAnnotationConfig();
+}
+
+void KMFolderCachedImap::setIncidencesFor( IncidencesFor incfor )
+{
+  if ( mIncidencesFor != incfor ) {
+    mIncidencesFor = incfor;
+    mIncidencesForChanged = true;
+  }
+}
+
+void KMFolderCachedImap::slotAnnotationResult(const QString& entry, const QString& value, bool found)
+{
+  if ( entry == KOLAB_FOLDERTYPE ) {
+    // There are four cases.
+    // 1) no content-type on server -> set it
+    // 2) different content-type on server, locally changed -> set it (we don't even come here)
+    // 3) different (known) content-type on server, no local change -> get it
+    // 4) different unknown content-type on server, probably some older version -> set it
+    if ( found ) {
+      QString type = value;
+      QString subtype;
+      int dot = value.find( '.' );
+      if ( dot != -1 ) {
+        type.truncate( dot );
+        subtype = value.mid( dot + 1 );
+      }
+      bool foundKnownType = false;
+      for ( uint i = 0 ; i <= ContentsTypeLast; ++i ) {
+        FolderContentsType contentsType = static_cast<KMail::FolderContentsType>( i );
+        if ( type == KMailICalIfaceImpl::annotationForContentsType( contentsType ) ) {
+          // Case 3: known content-type on server, get it
+          //kdDebug(5006) << mImapPath << ": slotGetAnnotationResult: found known type of annotation" << endl;
+          kmkernel->iCalIface().setStorageFormat( folder(), KMailICalIfaceImpl::StorageXML );
+          mAnnotationFolderType = value;
+          if ( folder()->parent()->owner()->idString() != GlobalSettings::theIMAPResourceFolderParent()
+               && GlobalSettings::theIMAPResourceEnabled()
+               && subtype == "default" ) {
+            // Truncate subtype if this folder can't be a default resource folder for us,
+            // although it apparently is for someone else.
+            mAnnotationFolderType = type;
+            kdDebug(5006) << mImapPath << ": slotGetAnnotationResult: parent folder is " << folder()->parent()->owner()->idString() << " => truncating annotation to " << value << endl;
+          }
+          setContentsType( contentsType );
+          mAnnotationFolderTypeChanged = false; // we changed it, not the user
+          foundKnownType = true;
+          // Ensure that further readConfig()s don't lose mAnnotationFolderType
+          writeAnnotationConfig();
+          break;
+        }
+      }
+      if ( !foundKnownType && !mReadOnly ) {
+        //kdDebug(5006) << "slotGetAnnotationResult: no known type of annotation found, will need to set it" << endl;
+        // Case 4: server has strange content-type, set it to what we need
+        mAnnotationFolderTypeChanged = true;
+      }
+      // TODO handle subtype (inbox, drafts, sentitems, junkemail)
+    }
+    else if ( !mReadOnly ) {
+      // Case 1: server doesn't have content-type, set it
+      //kdDebug(5006) << "slotGetAnnotationResult: no annotation found, will need to set it" << endl;
+      mAnnotationFolderTypeChanged = true;
+    }
+  } else if ( entry == KOLAB_INCIDENCESFOR ) {
+    if ( found ) {
+      mIncidencesFor = incidencesForFromString( value );
+      Q_ASSERT( mIncidencesForChanged == false );
+    }
+  }
+}
+
+void KMFolderCachedImap::slotGetAnnotationResult( KIO::Job* job )
+{
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  Q_ASSERT( it != mAccount->jobsEnd() );
+  if ( it == mAccount->jobsEnd() ) return; // Shouldn't happen
+  Q_ASSERT( (*it).parent == folder() );
+  if ( (*it).parent != folder() ) return; // Shouldn't happen
+
+  AnnotationJobs::GetAnnotationJob* annjob = static_cast<AnnotationJobs::GetAnnotationJob *>( job );
+  if ( annjob->error() ) {
+    if ( job->error() == KIO::ERR_UNSUPPORTED_ACTION ) {
+      // that's when the imap server doesn't support annotations
+      if ( GlobalSettings::theIMAPResourceStorageFormat() == GlobalSettings::EnumTheIMAPResourceStorageFormat::XML
+           && (uint)GlobalSettings::theIMAPResourceAccount() == mAccount->id() )
+	KMessageBox::error( 0, i18n( "The IMAP server %1 does not have support for IMAP annotations. The XML storage cannot be used on this server; please re-configure KMail differently." ).arg( mAccount->host() ) );
+      mAccount->setHasNoAnnotationSupport();
+    }
+    else
+      kdWarning(5006) << "slotGetAnnotationResult: " << job->errorString() << endl;
+  }
+
+  if (mAccount->slave()) mAccount->removeJob(job);
+  mProgress += 2;
+  serverSyncInternal();
+}
+
+void
+KMFolderCachedImap::slotAnnotationChanged( const QString& entry, const QString& attribute, const QString& value )
+{
+  kdDebug(5006) << k_funcinfo << entry << " " << attribute << " " << value << endl;
+  if ( entry == KOLAB_FOLDERTYPE )
+    mAnnotationFolderTypeChanged = false;
+  else if ( entry == KOLAB_INCIDENCESFOR ) {
+    mIncidencesForChanged = false;
+    // The incidences-for changed, we must trigger the freebusy creation.
+    // HACK: in theory we would need a new enum value for this.
+    kmkernel->iCalIface().addFolderChange( folder(), KMailICalIfaceImpl::ACL );
+  }
+}
+
+void KMFolderCachedImap::slotTestAnnotationResult(KIO::Job *job)
+{
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  Q_ASSERT( it != mAccount->jobsEnd() );
+  if ( it == mAccount->jobsEnd() ) return; // Shouldn't happen
+  Q_ASSERT( (*it).parent == folder() );
+  if ( (*it).parent != folder() ) return; // Shouldn't happen
+
+  mAccount->setAnnotationCheckPassed( true );
+  if ( job->error() ) {
+    kdDebug(5006) << "Test Annotation was not passed, disabling annotation support" << endl;
+    mAccount->setHasNoAnnotationSupport( );
+  } else {
+    kdDebug(5006) << "Test Annotation was passed   OK" << endl;
+  }
+  if (mAccount->slave()) mAccount->removeJob(job);
+  serverSyncInternal();
+}
+
+void
+KMFolderCachedImap::slotSetAnnotationResult(KIO::Job *job)
+{
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  if ( it == mAccount->jobsEnd() ) return; // Shouldn't happen
+  if ( (*it).parent != folder() ) return; // Shouldn't happen
+
+  bool cont = true;
+  if ( job->error() ) {
+    // Don't show error if the server doesn't support ANNOTATEMORE and this folder only contains mail
+    if ( job->error() == KIO::ERR_UNSUPPORTED_ACTION && contentsType() == ContentsTypeMail )
+      if (mAccount->slave()) mAccount->removeJob(job);
+    else
+      cont = mAccount->handleJobError( job, i18n( "Error while setting annotation: " ) + '\n' );
+  } else {
+    if (mAccount->slave()) mAccount->removeJob(job);
+  }
+  if ( cont )
+    serverSyncInternal();
 }
 
 void KMFolderCachedImap::slotUpdateLastUid()
@@ -1641,4 +2011,10 @@ void KMFolderCachedImap::slotUpdateLastUid()
   mTentativeHighestUid = 0;
 }
 
+bool KMFolderCachedImap::isMoveable() const
+{
+  return ( hasChildren() == HasNoChildren && 
+      !folder()->isSystemFolder() ) ? true : false;
+}
+
 #include "kmfoldercachedimap.moc"
