@@ -13,7 +13,7 @@
 #include "filterlog.h"
 using KMail::FilterLog;
 
-#include <libkdepim/email.h>
+#include <libemailfunctions/email.h>
 
 #include <kglobal.h>
 #include <klocale.h>
@@ -161,8 +161,6 @@ const QString KMSearchRule::asString() const
   return result;
 }
 
-
-
 //==================================================
 //
 // class KMSearchRuleString
@@ -239,7 +237,7 @@ bool KMSearchRuleString::matches( const DwString & aStr, KMMessage & msg,
     if ( endOfHeader == DwString::npos )
       endOfHeader = lfcrlf.FindIn( aStr, 0 );
     const DwString headers = ( endOfHeader == DwString::npos ) ? aStr : aStr.substr( 0, endOfHeader );
-    // In case the searched header is at the beginning, we have to prepend 
+    // In case the searched header is at the beginning, we have to prepend
     // a newline - see the comment in KMSearchRuleString constructor
     DwString fakedHeaders( "\n" );
     size_t start = headerField->FindIn( fakedHeaders.append( headers ), 0, false );
@@ -401,7 +399,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
     for( QStringList::ConstIterator it = addressList.begin();
          ( it != addressList.end() );
          ++it ) {
-      if ( !stdAb->findByEmail( KPIM::getEmailAddr( *it ) ).isEmpty() )
+      if ( !stdAb->findByEmail( KPIM::getEmailAddress( *it ) ).isEmpty() )
         return true;
     }
     return false;
@@ -414,7 +412,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
     for( QStringList::ConstIterator it = addressList.begin();
          ( it != addressList.end() );
          ++it ) {
-      if ( stdAb->findByEmail( KPIM::getEmailAddr( *it ) ).isEmpty() )
+      if ( stdAb->findByEmail( KPIM::getEmailAddress( *it ) ).isEmpty() )
         return true;
     }
     return false;
@@ -427,7 +425,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
 
     for( QStringList::ConstIterator it = addressList.begin();
       it != addressList.end(); ++it ) {
-        KABC::Addressee::List addresses = stdAb->findByEmail( KPIM::getEmailAddr( *it ) );
+        KABC::Addressee::List addresses = stdAb->findByEmail( KPIM::getEmailAddress( *it ) );
 
           for ( KABC::Addressee::List::Iterator itAd = addresses.begin(); itAd != addresses.end(); ++itAd )
               if ( (*itAd).hasCategory(category) )
@@ -444,7 +442,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
 
       for( QStringList::ConstIterator it = addressList.begin();
         it != addressList.end(); ++it ) {
-          KABC::Addressee::List addresses = stdAb->findByEmail( KPIM::getEmailAddr( *it ) );
+          KABC::Addressee::List addresses = stdAb->findByEmail( KPIM::getEmailAddress( *it ) );
 
             for ( KABC::Addressee::List::Iterator itAd = addresses.begin(); itAd != addresses.end(); ++itAd )
                 if ( (*itAd).hasCategory(category) )
@@ -586,37 +584,37 @@ KMMsgStatus KMSearchRuleStatus::statusFromEnglishName(
       const QString & aStatusString )
 {
   KMMsgStatus status = 0;
-  if ( ! aStatusString.compare("important") )
+  if ( ! aStatusString.compare("Important") )
     status = KMMsgStatusFlag;
-  if ( ! aStatusString.compare("new") )
+  if ( ! aStatusString.compare("New") )
     status = KMMsgStatusNew;
-  if ( ! aStatusString.compare("unread") )
+  if ( ! aStatusString.compare("Unread") )
     status = KMMsgStatusUnread | KMMsgStatusNew;
-  if ( ! aStatusString.compare("read") )
+  if ( ! aStatusString.compare("Read") )
     status = KMMsgStatusRead;
-  if ( ! aStatusString.compare("old") )
+  if ( ! aStatusString.compare("Old") )
     status = KMMsgStatusOld;
-  if ( ! aStatusString.compare("deleted") )
+  if ( ! aStatusString.compare("Deleted") )
     status = KMMsgStatusDeleted;
-  if ( ! aStatusString.compare("replied") )
+  if ( ! aStatusString.compare("Replied") )
     status = KMMsgStatusReplied;
-  if ( ! aStatusString.compare("forwarded") )
+  if ( ! aStatusString.compare("Forwarded") )
     status = KMMsgStatusForwarded;
-  if ( ! aStatusString.compare("queued") )
+  if ( ! aStatusString.compare("Queued") )
     status = KMMsgStatusQueued;
-  if ( ! aStatusString.compare("sent") )
+  if ( ! aStatusString.compare("Sent") )
     status = KMMsgStatusSent;
-  if ( ! aStatusString.compare("watched") )
+  if ( ! aStatusString.compare("Watched") )
     status = KMMsgStatusWatched;
-  if ( ! aStatusString.compare("ignored") )
+  if ( ! aStatusString.compare("Ignored") )
     status = KMMsgStatusIgnored;
-  if ( ! aStatusString.compare("todo") )
+  if ( ! aStatusString.compare("To Do") )
     status = KMMsgStatusTodo;
-  if ( ! aStatusString.compare("spam") )
+  if ( ! aStatusString.compare("Spam") )
     status = KMMsgStatusSpam;
-  if ( ! aStatusString.compare("ham") )
+  if ( ! aStatusString.compare("Ham") )
      status = KMMsgStatusHam;
-  if ( ! aStatusString.compare("has an attachment") )
+  if ( ! aStatusString.compare("Has Attachment") )
      status = KMMsgStatusHasAttach;
 
   return status;
@@ -688,7 +686,7 @@ KMSearchPattern::~KMSearchPattern()
 {
 }
 
-bool KMSearchPattern::matches( const KMMessage * msg ) const
+bool KMSearchPattern::matches( const KMMessage * msg, bool ignoreBody ) const
 {
   if ( isEmpty() )
     return true;
@@ -697,20 +695,22 @@ bool KMSearchPattern::matches( const KMMessage * msg ) const
   switch ( mOperator ) {
   case OpAnd: // all rules must match
     for ( it.toFirst() ; it.current() ; ++it )
-      if ( !(*it)->matches( msg ) )
-	return false;
+      if ( !((*it)->requiresBody() && ignoreBody) )
+        if ( !(*it)->matches( msg ) )
+          return false;
     return true;
   case OpOr:  // at least one rule must match
     for ( it.toFirst() ; it.current() ; ++it )
-      if ( (*it)->matches( msg ) )
-	return true;
+      if ( !((*it)->requiresBody() && ignoreBody) )
+        if ( (*it)->matches( msg ) )
+          return true;
     // fall through
   default:
     return false;
   }
 }
 
-bool KMSearchPattern::matches( const DwString & aStr ) const
+bool KMSearchPattern::matches( const DwString & aStr, bool ignoreBody ) const
 {
   if ( isEmpty() )
     return true;
@@ -720,20 +720,22 @@ bool KMSearchPattern::matches( const DwString & aStr ) const
   switch ( mOperator ) {
   case OpAnd: // all rules must match
     for ( it.toFirst() ; it.current() ; ++it )
-      if ( !(*it)->matches( aStr, msg ) )
-	return false;
+      if ( !((*it)->requiresBody() && ignoreBody) )
+        if ( !(*it)->matches( aStr, msg ) )
+          return false;
     return true;
   case OpOr:  // at least one rule must match
     for ( it.toFirst() ; it.current() ; ++it )
-      if ( (*it)->matches( aStr, msg ) )
-	return true;
+      if ( !((*it)->requiresBody() && ignoreBody) )
+        if ( (*it)->matches( aStr, msg ) )
+          return true;
     // fall through
   default:
     return false;
   }
 }
 
-bool KMSearchPattern::matches( Q_UINT32 serNum ) const
+bool KMSearchPattern::matches( Q_UINT32 serNum, bool ignoreBody ) const
 {
   if ( isEmpty() )
     return true;
@@ -750,14 +752,14 @@ bool KMSearchPattern::matches( Q_UINT32 serNum ) const
   if ( !opened )
     folder->open();
   KMMsgBase *msgBase = folder->getMsgBase(idx);
-  if (requiresBody()) {
+  if (requiresBody() && !ignoreBody) {
     bool unGet = !msgBase->isMessage();
     KMMessage *msg = folder->getMsg(idx);
-    res = matches( msg );
+    res = matches( msg, ignoreBody );
     if (unGet)
       folder->unGetMsg(idx);
   } else {
-    res = matches( folder->getDwString(idx) );
+    res = matches( folder->getDwString(idx), ignoreBody );
   }
   if ( !opened )
     folder->close();
