@@ -2,7 +2,7 @@
     kncomposer.cpp
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2004 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -168,7 +168,8 @@ void KNLineEditSpell::spellCheckerCorrected( const QString &old, const QString &
 KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &sig, const QString &unwraped, bool firstEdit, bool dislikesCopies, bool createCopy)
     : KMainWindow(0,"composerWindow"), r_esult(CRsave), a_rticle(a), s_ignature(sig), u_nwraped(unwraped),
       n_eeds8Bit(true), v_alidated(false), a_uthorDislikesMailCopies(dislikesCopies), e_xternalEdited(false), e_xternalEditor(0),
-      e_ditorTempfile(0), s_pellChecker(0), a_ttChanged(false)
+      e_ditorTempfile(0), s_pellChecker(0), a_ttChanged(false),
+      mFirstEdit( firstEdit )
 {
     mSpellingFilter = 0;
     spellLineEdit = false;
@@ -344,8 +345,10 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   setConfig(false);
 
   if (firstEdit) {   // now we place the cursor at the end of the quoted text / below the attribution line
-    if (knGlobals.configManager()->postNewsComposer()->cursorOnTop())
-      v_iew->e_dit->setCursorPosition(1,0);
+    if (knGlobals.configManager()->postNewsComposer()->cursorOnTop()) {
+      int numLines = knGlobals.configManager()->postNewsComposer()->intro().contains("%L");
+      v_iew->e_dit->setCursorPosition(numLines+1,0);
+    }
     else
       v_iew->e_dit->setCursorPosition(v_iew->e_dit->numLines()-1,0);
   } else
@@ -579,10 +582,16 @@ bool KNComposer::hasValidData()
                                        QString::null, i18n("&Send"),i18n("edit article","&Edit")) == KMessageBox::Yes))
         return false;
 
-    if (!followUp && (groupCount>2))
-      if (!(KMessageBox::warningYesNo( this, i18n("You are crossposting to more than two newsgroups.\nPlease use the \"Followup-To\" header to direct\nthe replies to your article into one group.\nDo you want to re-edit the article or send it anyway?"),
-                                       QString::null, i18n("&Send"),i18n("edit article","&Edit")) == KMessageBox::Yes))
+    if ( !followUp && groupCount > 2 ) {
+      if ( KMessageBox::warningYesNo( this,
+           i18n("You are crossposting to more than two newsgroups.\n"
+                "Please use the \"Followup-To\" header to direct the replies "
+                "to your article into one group.\n"
+                "Do you want to re-edit the article or send it anyway?"),
+           QString::null, i18n("&Send"), i18n("edit article","&Edit"), "missingFollowUpTo" )
+           != KMessageBox::Yes )
         return false;
+    }
 
     if (fupCount>12) {
       KMessageBox::sorry(this, i18n("You are directing replies to more than 12 newsgroups.\nPlease remove some newsgroups from the \"Followup-To\" header."));
@@ -696,9 +705,9 @@ bool KNComposer::hasValidData()
     if ( acc ) {
       KMime::Headers::Newsgroups *grps = a_rticle->newsgroups();
       KNGroup *grp = knGlobals.groupManager()->group( grps->firstGroup(), acc );
-      if ( grp && grp->identity() && grp->identity()->hasSigningKey() )
+      if (grp && grp->identity())
         signingKey = grp->identity()->signingKey();
-      else if ( acc->identity() && acc->identity()->hasSigningKey() )
+      else if (acc->identity())
         signingKey = acc->identity()->signingKey();
     }
 
@@ -822,9 +831,9 @@ bool KNComposer::applyChanges()
       if ( acc ) {
           KMime::Headers::Newsgroups *grps = a_rticle->newsgroups();
           KNGroup *grp = knGlobals.groupManager()->group( grps->firstGroup(), acc );
-          if ( grp && grp->identity() && grp->identity()->hasSigningKey() )
+          if (grp && grp->identity())
               signingKey = grp->identity()->signingKey();
-          else if ( acc->identity() && acc->identity()->hasSigningKey() )
+          else if (acc->identity())
               signingKey = acc->identity()->signingKey();
       }
       // now try to sign the article
@@ -1435,7 +1444,7 @@ void KNComposer::slotGroupsChanged(const QString &t)
   }
   v_iew->f_up2->insertItem("");
 
-  if(!currText.isEmpty())
+  if ( !currText.isEmpty() || !mFirstEdit ) // user might have cleared fup2 intentionally during last edit
     v_iew->f_up2->lineEdit()->setText(currText);
 }
 
@@ -1530,7 +1539,7 @@ void KNComposer::slotCancelEditor()
 }
 
 
-void KNComposer::slotAttachmentPopup(QListViewItem *it, const QPoint &p, int)
+void KNComposer::slotAttachmentPopup(KListView*, QListViewItem *it, const QPoint &p)
 {
   if(it)
     a_ttPopup->popup(p);
@@ -1646,10 +1655,8 @@ void KNComposer::slotSpellFinished()
   {
       if( spellLineEdit )
           slotSpellcheck();
-#if KDE_IS_VERSION( 3, 1, 90 )
       else if( status == KSpell::FinishedNoMisspellingsEncountered )
           KMessageBox::information( this, i18n("No misspellings encountered."));
-#endif
   }
 }
 
@@ -1889,8 +1896,8 @@ void KNComposer::ComposerView::showAttachmentView()
     connect(a_ttView, SIGNAL(clicked ( QListViewItem * )),
             parent(), SLOT(slotAttachmentSelected(QListViewItem*)));
 
-    connect(a_ttView, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
-            parent(), SLOT(slotAttachmentPopup(QListViewItem*, const QPoint&, int)));
+    connect(a_ttView, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),
+            parent(), SLOT(slotAttachmentPopup(KListView*, QListViewItem*, const QPoint&)));
     connect(a_ttView, SIGNAL(delPressed(QListViewItem*)),
             parent(), SLOT(slotAttachmentRemove(QListViewItem*)));
     connect(a_ttView, SIGNAL(doubleClicked(QListViewItem*)),
@@ -2469,7 +2476,7 @@ void KNComposer::Editor::slotCorrectWord()
 
 
 KNComposer::AttachmentView::AttachmentView(QWidget *parent, char *name)
- : QListView(parent, name)
+ : KListView(parent, name)
 {
   setFrameStyle(QFrame::WinPanel | QFrame::Sunken);  // match the QMultiLineEdit style
   addColumn(i18n("File"), 115);
@@ -2495,15 +2502,15 @@ void KNComposer::AttachmentView::keyPressEvent(QKeyEvent *e)
   if( (e->key()==Key_Delete) && (currentItem()) )
     emit(delPressed(currentItem()));
   else
-    QListView::keyPressEvent(e);
+    KListView::keyPressEvent(e);
 }
 
 
 //=====================================================================================
 
 
-KNComposer::AttachmentViewItem::AttachmentViewItem(QListView *v, KNAttachment *a) :
-  QListViewItem(v), attachment(a)
+KNComposer::AttachmentViewItem::AttachmentViewItem(KListView *v, KNAttachment *a) :
+  KListViewItem(v), attachment(a)
 {
   setText(0, a->name());
   setText(1, a->mimeType());
@@ -2635,3 +2642,5 @@ void KNComposer::AttachmentPropertiesDlg::slotMimeTypeTextChanged(const QString
 //--------------------------------
 
 #include "kncomposer.moc"
+
+// kate: space-indent on; indent-width 2;
