@@ -1,4 +1,4 @@
-/* vcal-conduitbase.cc                      KPilot
+/* KPilot
 **
 ** Copyright (C) 2002-3 by Reinhold Kainhofer
 ** Copyright (C) 2001 by Dan Pilone
@@ -30,7 +30,7 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-static const char *vcalconduitbase_id = "$Id: vcal-conduitbase.cc,v 1.70.2.1 2004/09/29 09:20:54 adridg Exp $";
+static const char *vcalconduitbase_id = "$Id: vcal-conduitbase.cc,v 1.78 2005/01/13 15:52:41 adridg Exp $";
 
 #include <options.h>
 
@@ -53,6 +53,10 @@ static const char *vcalconduitbase_id = "$Id: vcal-conduitbase.cc,v 1.70.2.1 200
 #include "vcalconduitSettings.h"
 
 
+#ifndef LIBKCAL_IS_VERSION
+#warning "Using an old version of libkcal with timezone bug."
+#define LIBKCAL_IS_VERSION(a,b,c) (0)
+#endif
 
 
 
@@ -196,7 +200,20 @@ error:
 	setConflictResolution(res);
 }
 
+#ifdef DEBUG
+static void listResources(KCal::CalendarResources *p)
+{
+	FUNCTIONSETUP;
+	KCal::CalendarResourceManager *manager = p->resourceManager();
 
+	DEBUGCONDUIT << fname << ": Resources in calendar:" << endl;
+	KCal::CalendarResourceManager::Iterator it;
+	for( it = manager->begin(); it != manager->end(); ++it )
+	{
+		DEBUGCONDUIT << fname << ": " << (*it)->resourceName() << endl;
+	}
+}
+#endif
 
 /* virtual */ bool VCalConduitBase::openCalendar()
 {
@@ -255,10 +272,9 @@ error:
 				<< "Calendar is local time: "
 				<< fCalendar->isLocalTime() << endl;
 #endif
-			emit logMessage(i18n("Using %1 time zone: %2")
-				.arg(fCalendar->isLocalTime() ?
-					i18n("non-local") : i18n("local"))
-				.arg(tz));
+			emit logMessage( fCalendar->isLocalTime() ?
+				i18n("Using local time zone: %1").arg(tz) :
+				i18n("Using non-local time zone: %1").arg(tz) );
 
 			KURL kurl(config()->calendarFile());
 			if(!KIO::NetAccess::download(config()->calendarFile(), fCalendarFile, 0L) &&
@@ -312,6 +328,9 @@ error:
 			DEBUGCONDUIT << "Using CalendarResource!" << endl;
 #endif
 			rescal = new KCal::CalendarResources( tz );
+#ifdef DEBUG
+			listResources(rescal);
+#endif
 			fCalendar = rescal;
 			if ( !fCalendar)
 			{
@@ -322,12 +341,13 @@ error:
 #if LIBKCAL_IS_VERSION(1,1,0)
 			rescal->readConfig();
 			rescal->load();
+#else
+#warning "Timezone bug is present."
 #endif
 			addSyncLogEntry(i18n("Syncing with standard calendar resource."));
-			emit logMessage(i18n("Using %1 time zone: %2")
-				.arg(fCalendar->isLocalTime() ?
-					i18n("non-local") : i18n("local"))
-				.arg(tz));
+			emit logMessage( fCalendar->isLocalTime() ?
+				i18n("Using local time zone: %1").arg(tz) :
+				i18n("Using non-local time zone: %1").arg(tz) );
 			break;
 		default:
 			break;
@@ -567,7 +587,7 @@ void VCalConduitBase::cleanup()
 					else {
 						KIO::NetAccess::removeTempFile(fCalendarFile);
 					}
-					QFile backup(fCalendarFile + "~");
+					QFile backup(fCalendarFile + CSL1("~"));
 					backup.remove();
 				}
 				break;
@@ -632,11 +652,20 @@ int VCalConduitBase::resolveConflict(KCal::Incidence*e, PilotAppCategory*de) {
 	if (getConflictResolution()==SyncAction::eAskUser)
 	{
 		// TODO: This is messed up!!!
-		return KMessageBox::warningYesNo(NULL,
-			i18n("The following item was modified both on the Pilot and on your PC:\nPC entry:\n\t")+e->summary()+i18n("\nPilot entry:\n\t")+getTitle(de)+
-				i18n("\n\nShould the Pilot entry overwrite the PC entry? If you select \"No\", the PC entry will overwrite the Pilot entry."),
-			i18n("Conflicting Entries")
-		)==KMessageBox::No;
+		QString query = i18n("The following item was modified "
+			"both on the Handheld and on your PC:\nPC entry:\n\t");
+		query += e->summary();
+		query += i18n("\nHandheld entry:\n\t");
+		query += getTitle(de);
+		query += i18n("\n\nWhich entry do you want to keep? It will "
+			"overwrite the other entry.");
+
+		return KMessageBox::No == questionYesNo(
+			query,
+			i18n("Conflicting Entries"),
+			QString::null,
+			0 /* Never timeout */,
+			i18n("Handheld"), i18n("PC"));
 	}
 	return getConflictResolution();
 }
