@@ -1,6 +1,7 @@
-/* kpilotConfig.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This is all of KPilot's config-handling stuff.
 */
@@ -34,7 +35,7 @@
 #include <qcombobox.h>
 #include <qcheckbox.h>
 
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <kconfig.h>
 #include <ksimpleconfig.h>
 #include <kcmdlineargs.h>
@@ -45,7 +46,7 @@
 #include "kpilotConfig.h"
 
 static const char kpilotconfig_id[] =
-	"$Id: kpilotConfig.cc,v 1.36.2.1 2004/10/06 14:41:54 thorsen Exp $";
+	"$Id: kpilotConfig.cc,v 1.41 2005/01/13 15:52:41 adridg Exp $";
 
 // This is a number indicating what configuration version
 // we're dealing with. Whenever new configuration options are
@@ -236,7 +237,7 @@ static void update440()
 	const char **s = oldconduits;
 	while (*s)
 	{
-		QString libname = CSL1("kde3/lib%1conduit.so").arg(*s);
+		QString libname = CSL1("kde3/lib%1conduit.so").arg(QString::fromLatin1(*s));
 		QString foundlib = ::locate("lib",libname);
 		if (!foundlib.isEmpty())
 		{
@@ -265,7 +266,7 @@ static void update443()
 	bool fixedSome = false;
 #ifdef DEBUG
 	DEBUGKPILOT << fname << ": Skip databases are: "
-		<< skip.join(",") << endl;
+		<< skip.join(CSL1(",")) << endl;
 #endif
 
 	for (QStringList::const_iterator i = skip.begin(); i!=skip.end(); ++i)
