@@ -3,6 +3,7 @@
 /* pilotDaemon.h			KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** See the .cc file for an explanation of what this file is for.
 */
@@ -168,6 +169,7 @@ public:
 	virtual int nextSyncType() const;
 	virtual ASYNC quitNow();
 	virtual ASYNC reloadSettings();
+	virtual ASYNC setTempDevice(QString d);
 
 	virtual void stopListening();
 	virtual void startListening();
@@ -260,6 +262,7 @@ private:
 	LoggerDCOP_stub *fLogStub;
 	LoggerDCOP_stub *fLogFileStub;
 	KPilotDCOP_stub *fKPilotStub;
+	QString fTempDevice;
 };
 
 
