@@ -44,6 +44,7 @@
 #include <libkcal/todo.h>
 #include <libkcal/journal.h>
 #include <libkcal/filestorage.h>
+#include <libkcal/confirmsavedialog.h>
 
 #include <kabc/locknull.h>
 #include <kabc/stdaddressbook.h>
@@ -53,7 +54,6 @@
 #include "webdavhandler.h"
 #include "kcalsloxprefs.h"
 #include "sloxaccounts.h"
-#include "confirmsavedialog.h"
 
 #include "kcalresourceslox.h"
 
@@ -64,6 +64,8 @@ KCalResourceSlox::KCalResourceSlox( const KConfig *config )
 {
   init();
 
+  mPrefs->addGroupPrefix( identifier() );
+
   if ( config ) {
     readConfig( config );
   }
@@ -74,6 +76,8 @@ KCalResourceSlox::KCalResourceSlox( const KURL &url )
 {
   init();
 
+  mPrefs->addGroupPrefix( identifier() );
+
   mPrefs->setUrl( url.url() );
 }
 
@@ -110,8 +114,6 @@ void KCalResourceSlox::init()
 
   setType( "slox" );
 
-  mOpen = false;
-
   mLock = new KABC::LockNull( true );
 
   enableChangeNotification();
@@ -144,31 +146,10 @@ void KCalResourceSlox::writeConfig( KConfig *config )
   ResourceCached::writeConfig( config );
 }
 
-QString KCalResourceSlox::cacheFile()
-{
-  QString file = locateLocal( "cache", "kcal/kresources/" + identifier() );
-  kdDebug() << "KCalResourceSlox::cacheFile(): " << file << endl;
-  return file;
-}
-
-bool KCalResourceSlox::doOpen()
-{
-  kdDebug(5800) << "KCalResourceSlox::doOpen()" << endl;
-
-  mOpen = true;
-
-  return true;
-}
-
 bool KCalResourceSlox::doLoad()
 {
   kdDebug() << "KCalResourceSlox::load() " << int( this ) << endl;
 
-  if ( !mOpen ) {
-    kdWarning() << "Warning: resource not open." << endl;
-    return true;
-  }
-
   if ( mLoadEventsJob || mLoadTodosJob ) {
     kdWarning() << "KCalResourceSlox::load(): download still in progress."
                 << endl;
@@ -185,7 +166,7 @@ bool KCalResourceSlox::doLoad()
   mCalendar.close();
 
   disableChangeNotification();
-  mCalendar.load( cacheFile() );
+  loadCache();
   enableChangeNotification();
 
   emit resourceChanged( this );
@@ -286,7 +267,7 @@ void KCalResourceSlox::requestTodos()
            SLOT( slotTodosProgress( KIO::Job *, unsigned long ) ) );
 
   mLoadTodosProgress = KPIM::ProgressManager::instance()->createProgressItem(
-      KPIM::ProgressManager::getUniqueID(), i18n("Downloading todos") );
+      KPIM::ProgressManager::getUniqueID(), i18n("Downloading to-dos") );
   connect( mLoadTodosProgress,
            SIGNAL( progressItemCanceled( ProgressItem * ) ),
            SLOT( cancelLoadTodos() ) );
@@ -854,7 +835,7 @@ void KCalResourceSlox::slotLoadEventsResult( KIO::Job *job )
 
     enableChangeNotification();
 
-    mCalendar.save( cacheFile() );
+    saveCache();
 
     clearChanges();
 
@@ -953,7 +934,7 @@ void KCalResourceSlox::slotUploadResult( KIO::Job *job )
           disableChangeNotification();
           mCalendar.deleteIncidence( mUploadedIncidence );
           mCalendar.addIncidence( i );
-          mCalendar.save( cacheFile() );
+          saveCache();
           enableChangeNotification();
 
           emit resourceChanged( this );
@@ -1024,8 +1005,6 @@ bool KCalResourceSlox::doSave()
 {
   kdDebug() << "KCalResourceSlox::save()" << endl;
 
-  if ( !mOpen ) return true;
-
   if ( readOnly() || !hasChanges() ) {
     emit resourceSaved( this );
     return true;
@@ -1044,7 +1023,7 @@ bool KCalResourceSlox::doSave()
 
   if ( !confirmSave() ) return false;
 
-  mCalendar.save( cacheFile() );
+  saveCache();
 
   uploadIncidences();
 
@@ -1060,8 +1039,6 @@ void KCalResourceSlox::doClose()
 {
   kdDebug() << "KCalResourceSlox::doClose()" << endl;
 
-  if ( !mOpen ) return;
-
   cancelLoadEvents();
   cancelLoadTodos();
 
@@ -1070,8 +1047,6 @@ void KCalResourceSlox::doClose()
   } else {
     mCalendar.close();
   }
-
-  mOpen = false;
 }
 
 KABC::Lock *KCalResourceSlox::lock()
