@@ -54,18 +54,18 @@
 using namespace KPIM;
 
 KCompletion * AddresseeLineEdit::s_completion = 0L;
+KPIM::CompletionItemsMap* AddresseeLineEdit::s_completionItemMap = 0L;
 bool AddresseeLineEdit::s_addressesDirty = false;
 QTimer* AddresseeLineEdit::s_LDAPTimer = 0L;
 KPIM::LdapSearch* AddresseeLineEdit::s_LDAPSearch = 0L;
 QString* AddresseeLineEdit::s_LDAPText = 0L;
 AddresseeLineEdit* AddresseeLineEdit::s_LDAPLineEdit = 0L;
-KConfig *AddresseeLineEdit::s_config = 0L;
 
 static KStaticDeleter<KCompletion> completionDeleter;
+static KStaticDeleter<KPIM::CompletionItemsMap> completionItemsDeleter;
 static KStaticDeleter<QTimer> ldapTimerDeleter;
 static KStaticDeleter<KPIM::LdapSearch> ldapSearchDeleter;
 static KStaticDeleter<QString> ldapTextDeleter;
-static KStaticDeleter<KConfig> configDeleter;
 
 // needs to be unique, but the actual name doesn't matter much
 static QCString newLineEditDCOPObjectName()
@@ -86,6 +86,7 @@ AddresseeLineEdit::AddresseeLineEdit( QWidget* parent, bool useCompletion,
   m_useCompletion = useCompletion;
   m_completionInitialized = false;
   m_smartPaste = false;
+  m_addressBookConnected = false;
 
   init();
 
@@ -100,6 +101,8 @@ void AddresseeLineEdit::init()
     completionDeleter.setObject( s_completion, new KCompletion() );
     s_completion->setOrder( KCompletion::Weighted );
     s_completion->setIgnoreCase( true );
+
+    completionItemsDeleter.setObject( s_completionItemMap, new KPIM::CompletionItemsMap() );
   }
 
 //  connect( s_completion, SIGNAL( match( const QString& ) ),
@@ -212,15 +215,13 @@ void AddresseeLineEdit::insert( const QString &t )
 
   QString contents = text();
   int start_sel = 0;
-  int end_sel = 0;
   int pos = cursorPosition();
-  if ( getSelection( &start_sel, &end_sel ) ) {
+
+  if ( hasSelectedText() ) {
     // Cut away the selection.
-    if ( pos > end_sel )
-      pos -= (end_sel - start_sel);
-    else if ( pos > start_sel )
+    start_sel = selectionStart();
       pos = start_sel;
-    contents = contents.left( start_sel ) + contents.right( end_sel + 1 );
+    contents = contents.left( start_sel ) + contents.mid( start_sel + selectedText().length() );
   }
 
   int eot = contents.length();
@@ -237,6 +238,7 @@ void AddresseeLineEdit::insert( const QString &t )
 
   contents = contents.left( pos ) + newText + contents.mid( pos );
   setText( contents );
+  setEdited( true );
   setCursorPosition( pos + newText.length() );
 }
 
@@ -299,7 +301,7 @@ void AddresseeLineEdit::dropEvent( QDropEvent *e )
   }
 
   if ( m_useCompletion )
-     m_smartPaste = true;
+    m_smartPaste = true;
   QLineEdit::dropEvent( e );
   m_smartPaste = false;
 }
@@ -319,35 +321,17 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
   if ( !m_useCompletion )
     return;
 
-  QString prevAddr;
-
-  QString s( text() );
-  int n = s.findRev(',');
-
-  if ( n >= 0 ) {
-    n++; // Go past the ","
-
-    int len = s.length();
-
-    // Increment past any whitespace...
-    while ( n < len && s[ n ].isSpace() )
-      n++;
-
-    prevAddr = s.left( n );
-    s = s.mid( n, 255 ).stripWhiteSpace();
-  }
-
   if ( s_addressesDirty )
     loadContacts(); // read from local address book
 
   // cursor at end of string - or Ctrl+T pressed for substring completion?
   if ( ctrlT ) {
-    const QStringList completions = s_completion->substringCompletion( s );
+    const QStringList completions = s_completion->substringCompletion( m_searchString );
 
-    if ( completions.count() > 1 )
-      m_previousAddresses = prevAddr;
-    else if ( completions.count() == 1 )
-      setText( prevAddr + completions.first() );
+    if ( completions.count() == 1 ) {
+      setText( m_searchString + completions.first() );
+      setEdited( true );
+    }
 
     setCompletedItems( completions, true ); // this makes sure the completion popup is closed if no matching items were found
 
@@ -360,19 +344,19 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
   switch ( mode ) {
     case KGlobalSettings::CompletionPopupAuto:
     {
-      if ( s.isEmpty() )
+      if ( m_searchString.isEmpty() )
         break;
     }
 
     case KGlobalSettings::CompletionPopup:
     {
-      m_previousAddresses = prevAddr;
-      QStringList items = s_completion->allMatches( s );
-      items += s_completion->allMatches( "\"" + s );
+      //m_previousAddresses = prevAddr;
+      QStringList items = s_completion->allMatches( m_searchString );
+      items += s_completion->allMatches( "\"" + m_searchString );
       uint beforeDollarCompletionCount = items.count();
 
-      if ( s.find( ' ' ) == -1 ) // one word, possibly given name
-        items += s_completion->allMatches( "$$" + s );
+      if ( m_searchString.find( ' ' ) == -1 ) // one word, possibly given name
+        items += s_completion->allMatches( "$$" + m_searchString );
 
       if ( items.isEmpty() ) {
         setCompletedItems( items, false );
@@ -391,8 +375,8 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
         setCompletedItems( items, autoSuggest );
 
         if ( !autoSuggest ) {
-          int index = items.first().find( s );
-          QString newText = prevAddr + items.first().mid( index );
+          int index = items.first().find( m_searchString );
+          QString newText = m_previousAddresses + items.first().mid( index );
           setUserSelection( false );
           setCompletedText( newText, true );
         }
@@ -403,9 +387,10 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
 
     case KGlobalSettings::CompletionShell:
     {
-      QString match = s_completion->makeCompletion( s );
-      if ( !match.isNull() && match != s ) {
-        setText( prevAddr + match );
+      QString match = s_completion->makeCompletion( m_searchString );
+      if ( !match.isNull() && match != m_searchString ) {
+        setText( m_previousAddresses + match );
+        setEdited( true );
         cursorAtEnd();
       }
       break;
@@ -414,10 +399,10 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
     case KGlobalSettings::CompletionMan: // Short-Auto in fact
     case KGlobalSettings::CompletionAuto:
     {
-      if ( !s.isEmpty() ) {
-        QString match = s_completion->makeCompletion( s );
-        if ( !match.isNull() && match != s ) {
-          QString adds = prevAddr + match;
+      if ( !m_searchString.isEmpty() ) {
+        QString match = s_completion->makeCompletion( m_searchString );
+        if ( !match.isNull() && match != m_searchString ) {
+          QString adds = m_previousAddresses + match;
           setCompletedText( adds );
         }
         break;
@@ -433,6 +418,7 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
 void AddresseeLineEdit::slotPopupCompletion( const QString& completion )
 {
   setText( m_previousAddresses + completion );
+  setEdited( true );
   cursorAtEnd();
 //  slotMatched( m_previousAddresses + completion );
 }
@@ -484,10 +470,10 @@ void AddresseeLineEdit::loadContacts()
 
   QApplication::restoreOverrideCursor();
 
-  disconnect( addressBook, SIGNAL( addressBookChanged( AddressBook* ) ),
-              this, SLOT( loadContacts() ) );
-
-  connect( addressBook, SIGNAL( addressBookChanged( AddressBook* ) ), SLOT( loadContacts() ) );
+  if ( !m_addressBookConnected ) {
+    connect( addressBook, SIGNAL( addressBookChanged( AddressBook* ) ), SLOT( loadContacts() ) );
+    m_addressBookConnected = true;
+  }
 }
 
 void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight )
@@ -501,13 +487,68 @@ void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight )
     if( '\0' == (*it)[len-1] )
       --len;
     const QString tmp = (*it).left( len );
-    //kdDebug(5300) << "     AddresseeLineEdit::addContact() \"" << tmp << "\"" << endl;
-    QString fullEmail = addr.fullEmail( tmp );
-    //kdDebug(5300) << "                                     \"" << fullEmail << "\"" << endl;
-    s_completion->addItem( fullEmail.simplifyWhiteSpace(), weight );
+    const QString fullEmail = addr.fullEmail( tmp );
+    //kdDebug(5300) << "AddresseeLineEdit::addContact() \"" << fullEmail << "\" weight=" << weight << endl;
+    addCompletionItem( fullEmail.simplifyWhiteSpace(), weight );
+    // Try to guess the last name: if found, we add an extra
+    // entry to the list to make sure completion works even
+    // if the user starts by typing in the last name.
+    QString name( addr.realName().simplifyWhiteSpace() );
+    if( name.endsWith("\"") )
+      name.truncate( name.length()-1 );
+    if( name.startsWith("\"") )
+      name = name.mid( 1 );
+
+    // While we're here also add "email (full name)" for completion on the email
+    if ( !name.isEmpty() )
+      addCompletionItem( addr.preferredEmail() + " (" + name + ")", weight );
+
+    bool bDone = false;
+    int i = 1;
+    do{
+      i = name.findRev(' ');
+      if( 1 < i ){
+        QString sLastName( name.mid(i+1) );
+        if( ! sLastName.isEmpty() &&
+            2 <= sLastName.length() &&   // last names must be at least 2 chars long
+            ! sLastName.endsWith(".") ){ // last names must not end with a dot (like "Jr." or "Sr.")
+          name.truncate( i );
+          if( !name.isEmpty() ){
+            sLastName.prepend( "\"" );
+            sLastName.append( ", " + name + "\" <" );
+          }
+          QString sExtraEntry( sLastName );
+          sExtraEntry.append( tmp.isEmpty() ? addr.preferredEmail() : tmp );
+          sExtraEntry.append( ">" );
+          //kdDebug(5300) << "AddresseeLineEdit::addContact() added extra \"" << sExtraEntry.simplifyWhiteSpace() << "\" weight=" << weight << endl;
+          addCompletionItem( sExtraEntry.simplifyWhiteSpace(), weight );
+          bDone = true;
+        }
+      }
+      if( !bDone ){
+        name.truncate( i );
+        if( name.endsWith("\"") )
+          name.truncate( name.length()-1 );
+      }
+    }while( 1 < i && !bDone );
   }
 }
 
+void AddresseeLineEdit::addCompletionItem( const QString& string, int weight )
+{
+  // Check if there is an exact match for item already, and use the max weight if so.
+  // Since there's no way to get the information from KCompletion, we have to keep our own QMap
+  CompletionItemsMap::iterator it = s_completionItemMap->begin();
+  if ( it != s_completionItemMap->end() ) {
+    weight = QMAX( *it, weight );
+    *it = weight;
+  } else {
+    s_completionItemMap->insert( string, weight );
+  }
+
+  s_completion->addItem( string, weight );
+}
+
 void AddresseeLineEdit::slotStartLDAPLookup()
 {
   if ( !s_LDAPSearch->isAvailable() || s_LDAPLineEdit != this )
@@ -520,6 +561,7 @@ void AddresseeLineEdit::stopLDAPLookup()
 {
   s_LDAPSearch->cancelSearch();
   s_LDAPLineEdit = NULL;
+  s_completionItemMap->clear();
 }
 
 void AddresseeLineEdit::startLoadingLDAPEntries()
@@ -559,20 +601,36 @@ void AddresseeLineEdit::slotLDAPSearchData( const KPIM::LdapResultList& adrs )
 
 void AddresseeLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
 {
-    QString txt = text();
+    KCompletionBox* completionBox = this->completionBox();
 
     if ( !items.isEmpty() &&
-         !(items.count() == 1 && txt == items.first()) )
+         !(items.count() == 1 && m_searchString == items.first()) )
     {
-        if ( !txt.isEmpty() )
-            completionBox()->setCancelledText( txt );
-
-        completionBox()->setItems( items );
-        completionBox()->popup();
+        if ( completionBox->isVisible() )
+        {
+          bool wasSelected = completionBox->isSelected( completionBox->currentItem() );
+          const QString currentSelection = completionBox->currentText();
+          completionBox->setItems( items );
+          QListBoxItem* item = completionBox->findItem( currentSelection, Qt::ExactMatch );
+          if ( item )
+          {
+            completionBox->blockSignals( true );
+            completionBox->setCurrentItem( item );
+            completionBox->setSelected( item, wasSelected );
+            completionBox->blockSignals( false );
+          }
+        }
+        else // completion box not visible yet -> show it
+        {
+          if ( !m_searchString.isEmpty() )
+            completionBox->setCancelledText( m_searchString );
+          completionBox->setItems( items );
+          completionBox->popup();
+        }
 
         if ( autoSuggest )
         {
-            int index = items.first().find( txt );
+            int index = items.first().find( m_searchString );
             QString newText = items.first().mid( index );
             setUserSelection(false);
             setCompletedText(newText,true);
@@ -580,8 +638,8 @@ void AddresseeLineEdit::setCompletedItems( const QStringList& items, bool autoSu
     }
     else
     {
-        if ( completionBox() && completionBox()->isVisible() )
-            completionBox()->hide();
+        if ( completionBox && completionBox->isVisible() )
+            completionBox->hide();
     }
 }
 
@@ -604,15 +662,6 @@ void AddresseeLineEdit::slotEditCompletionOrder()
   editor.exec();
 }
 
-KConfig* AddresseeLineEdit::config()
-{
-  if ( !s_config )
-    configDeleter.setObject( s_config, new KConfig( locateLocal( "config",
-                             "kabldaprc" ) ) );
-
-  return s_config;
-}
-
 void KPIM::AddresseeLineEdit::slotIMAPCompletionOrderChanged()
 {
   if ( m_useCompletion )
@@ -626,4 +675,28 @@ void KPIM::AddresseeLineEdit::slotUserCancelled( const QString& cancelText )
   userCancelled( cancelText ); // in KLineEdit
 }
 
+void KPIM::AddresseeLineEdit::slotCompletion()
+{
+  // Called by KLineEdit's keyPressEvent -> new text, update search string
+  m_searchString = text();
+  int n = m_searchString.findRev(',');
+  if ( n >= 0 ) {
+    ++n; // Go past the ","
+
+    int len = m_searchString.length();
+
+    // Increment past any whitespace...
+    while ( n < len && m_searchString[ n ].isSpace() )
+      ++n;
+
+    m_previousAddresses = m_searchString.left( n );
+    m_searchString = m_searchString.mid( n ).stripWhiteSpace();
+  }
+  else
+  {
+    m_previousAddresses = QString::null;
+  }
+  doCompletion( false );
+}
+
 #include "addresseelineedit.moc"
